@@ -7,7 +7,7 @@
     Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -38,6 +38,14 @@ class Filter
 {
     public:
         Filter(FilterParams *pars_, SynthEngine *_synth);
+        Filter(const Filter &orig) :
+            pars(orig.pars),
+            parsUpdate(orig.parsUpdate),
+            category(orig.category),
+            synth(orig.synth)
+        {
+            filter = orig.filter->clone();
+        };
         ~Filter();
         void filterout(float *smp);
         void setfreq(float frequency);
