@@ -7,7 +7,7 @@
     Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -27,14 +27,18 @@
 #define SV_FILTER_H
 
 #include "DSP/Filter_.h"
+#include "Misc/Alloc.h"
+
 
 class SynthEngine;
 
 class SVFilter : public Filter_
 {
     public:
+       ~SVFilter() = default;
         SVFilter(unsigned char Ftype, float Ffreq, float Fq, unsigned char Fstages, SynthEngine *_synth);
-        ~SVFilter();
+        SVFilter(const SVFilter &orig);
+        Filter_* clone() { return new SVFilter(*this); };
         void filterout(float *smp);
         void setfreq(float frequency);
         void setfreq_and_q(float frequency, float q_);
@@ -63,7 +67,7 @@ class SVFilter : public Filter_
         int abovenq;   // this is 1 if the frequency is above the nyquist
         int oldabovenq;
         int needsinterpolation, firsttime;
-        float *tmpismp; // used if it needs interpolation in filterout()
+        Samples tmpismp; // used if it needs interpolation in filterout()
 
         SynthEngine *synth;
 };
