@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2019, Will Godfrey
+    Copyright 2014-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -25,8 +25,12 @@
 */
 
 #include "Misc/SynthEngine.h"
+#include "Misc/NumericFuncs.h"
 #include "Effects/Chorus.h"
 
+using func::power;
+
+
 #define MAX_CHORUS_DELAY 250.0f // ms
 
 static const int PRESET_SIZE = 12;
@@ -56,9 +60,9 @@ static unsigned char presets[NUM_PRESETS][PRESET_SIZE] = {
 
 
 Chorus::Chorus(bool insertion_, float *const efxoutl_, float *efxoutr_, SynthEngine *_synth) :
-    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0),
+    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0, _synth),
     lfo(_synth),
-    synth(_synth)
+    fb(0, _synth->samplerate)
 {
     dlk = drk = 0;
     maxdelay = (int)(MAX_CHORUS_DELAY / 1000.0f * synth->samplerate_f);
@@ -93,6 +97,8 @@ float Chorus::getdelay(float xlfo)
 // Apply the effect
 void Chorus::out(float *smpsl, float *smpsr)
 {
+    outvolume.advanceValue(synth->sent_buffersize);
+
     const float one = 1.0f;
     dl1 = dl2;
     dr1 = dr2;
@@ -161,8 +167,11 @@ void Chorus::out(float *smpsl, float *smpsr)
 // Cleanup the effect
 void Chorus::cleanup(void)
 {
+    Effect::cleanup();
+    fb.pushToTarget();
     for (int i = 0; i < maxdelay; ++i)
         delayl[i] = delayr[i] = 0.0f;
+    lfo.resetState();
 }
 
 
@@ -170,14 +179,14 @@ void Chorus::cleanup(void)
 void Chorus::setdepth(unsigned char Pdepth_)
 {
     Pdepth = Pdepth_;
-    depth = (powf(8.0f, (Pdepth / 127.0f) * 2.0f) - 1.0f) / 1000.0f; // seconds
+    depth = (power<8>((Pdepth / 127.0f) * 2.0f) - 1.0f) / 1000.0f; // seconds
 }
 
 
 void Chorus::setdelay(unsigned char Pdelay_)
 {
     Pdelay = Pdelay_;
-    delay = (powf(10.0f, (Pdelay / 127.0f) * 2.0f) - 1.0f) / 1000.0f; // seconds
+    delay = (power<10>((Pdelay / 127.0f) * 2.0f) - 1.0f) / 1000.0f; // seconds
 }
 
 
@@ -192,7 +201,7 @@ void Chorus::setvolume(unsigned char Pvolume_)
 {
     Pvolume = Pvolume_;
     outvolume.setTargetValue(Pvolume / 127.0f);
-    volume.setTargetValue((!insertion) ? 1.0f : outvolume.getValue());
+    volume.setTargetValue((!insertion) ? 1.0f : outvolume.getTargetValue());
 }
 
 
@@ -204,6 +213,8 @@ void Chorus::setpreset(unsigned char npreset)
             npreset = NUM_PRESETS - 1;
         for (int n = 0; n < PRESET_SIZE; ++n)
             changepar(n, presets[npreset][n]);
+        // All presets use no BPM syncing.
+        changepar(EFFECT::control::bpm, 0);
         Ppreset = npreset;
     }
     else
@@ -270,8 +281,15 @@ void Chorus::changepar(int npar, unsigned char value)
         case 11:
             Poutsub = (value > 1) ? 1 : value;
             break;
+        case EFFECT::control::bpm:
+            lfo.Pbpm = value;
+            break;
+        case EFFECT::control::bpmStart:
+            lfo.PbpmStart = value;
+            break;
         default:
             Pchanged = false;
+            break;
     }
 }
 
@@ -293,6 +311,8 @@ unsigned char Chorus::getpar(int npar)
         case 9:  return Plrcross;
         case 10: return Pflangemode;
         case 11: return Poutsub;
+        case EFFECT::control::bpm: return lfo.Pbpm;
+        case EFFECT::control::bpmStart: return lfo.PbpmStart;
         default: return 0;
     }
 }
@@ -300,9 +320,9 @@ unsigned char Chorus::getpar(int npar)
 
 float Choruslimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value.F;
+    int value = getData->data.value;
     int control = getData->data.control;
-    int request = getData->data.type & 3; // clear upper bits
+    int request = getData->data.type & TOPLEVEL::type::Default; // clear flags
     int npart = getData->data.part;
     int presetNum = getData->data.engine;
     int min = 0;
@@ -341,7 +361,13 @@ float Choruslimit::getlimits(CommandBlock *getData)
             max = 1;
             canLearn = 0;
             break;
-        case 16:
+        case EFFECT::control::bpm:
+            max = 1;
+            canLearn = 0;
+            break;
+        case EFFECT::control::bpmStart:
+            break;
+        case EFFECT::control::preset:
             max = 9;
             canLearn = 0;
             break;
@@ -354,9 +380,9 @@ float Choruslimit::getlimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
             break;
         case TOPLEVEL::type::Minimum:
