@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2018, Will Godfrey
+    Copyright 2018-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -22,7 +22,7 @@
 
     This file is a derivative of a ZynAddSubFX original.
 
-    Modified February 2018
+    Modified March 2019
 */
 
 #ifndef CHORUS_H
@@ -47,6 +47,7 @@ class Chorus : public Effect
 
     private:
         // Chorus Parameters
+        bool Pchanged;
         unsigned char Pvolume;
         unsigned char Pdepth;      // the depth of the Chorus(ms)
         unsigned char Pdelay;      // the delay (ms)
@@ -66,7 +67,7 @@ class Chorus : public Effect
         // Internal Values
         float depth;
         float delay;
-        InterpolatedParameter fb;
+        synth::InterpolatedValue<float> fb;
         float dl1;
         float dl2;
         float dr1;
@@ -83,8 +84,6 @@ class Chorus : public Effect
         int dlhi2;
         float dllo;
         float mdel;
-
-        SynthEngine *synth;
 };
 class Choruslimit
 {
