@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2019, Will Godfrey
+    Copyright 2014-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -43,15 +43,14 @@ static const char presets[NUM_PRESETS][PRESET_SIZE] = {
 };
 
 DynamicFilter::DynamicFilter(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
-    Effect(insertion_, efxoutl_, efxoutr_, new FilterParams(0, 64, 64, 0, _synth), 0),
+    Effect(insertion_, efxoutl_, efxoutr_, new FilterParams(0, 64, 64, 0, _synth), 0, _synth),
     lfo(_synth),
     Pdepth(0),
     Pampsns(90),
     Pampsnsinv(0),
     Pampsmooth(60),
     filterl(NULL),
-    filterr(NULL),
-    synth(_synth)
+    filterr(NULL)
 {
     setvolume(110);
     setpreset(Ppreset);
@@ -72,6 +71,8 @@ DynamicFilter::~DynamicFilter()
 // Apply the effect
 void DynamicFilter::out(float *smpsl, float *smpsr)
 {
+    outvolume.advanceValue(synth->sent_buffersize);
+
     if (filterpars->changed)
     {
         filterpars->changed = false;
@@ -120,8 +121,10 @@ void DynamicFilter::out(float *smpsl, float *smpsr)
 // Cleanup the effect
 void DynamicFilter::cleanup(void)
 {
+    Effect::cleanup();
     reinitfilter();
     ms1 = ms2 = ms3 = ms4 = 0.0f;
+    lfo.resetState();
 }
 
 
@@ -168,9 +171,7 @@ void DynamicFilter::reinitfilter(void)
 
 void DynamicFilter::setpreset(unsigned char npreset)
 {
-
-
-if (npreset < 0xf)
+    if (npreset < 0xf)
     {
         if (npreset >= NUM_PRESETS)
             npreset = NUM_PRESETS - 1;
@@ -270,6 +271,9 @@ if (npreset < 0xf)
         if (insertion == 0)
             changepar(0, presets[npreset][0] * 0.5f); // lower the volume if this is
                                                   // system effect
+        // All presets use no BPM syncing.
+        changepar(EFFECT::control::bpm, 0);
+
         Ppreset = npreset;
         reinitfilter();
     }
@@ -341,6 +345,14 @@ void DynamicFilter::changepar(int npar, unsigned char value)
             Pampsmooth = value;
             setampsns(Pampsns);
             break;
+
+        case EFFECT::control::bpm:
+            lfo.Pbpm = value;
+            break;
+
+        case EFFECT::control::bpmStart:
+            lfo.PbpmStart = value;
+            break;
     }
     Pchanged = true;
 }
@@ -361,6 +373,8 @@ unsigned char DynamicFilter::getpar(int npar)
         case 7:  return Pampsns;
         case 8:  return Pampsnsinv;
         case 9:  return Pampsmooth;
+        case EFFECT::control::bpm: return lfo.Pbpm;
+        case EFFECT::control::bpmStart: return lfo.PbpmStart;
         default: break;
     }
     return 0;
@@ -369,9 +383,9 @@ unsigned char DynamicFilter::getpar(int npar)
 
 float Dynamlimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value.F;
+    int value = getData->data.value;
     int control = getData->data.control;
-    int request = getData->data.type & 3; // clear upper bits
+    int request = getData->data.type & TOPLEVEL::type::Default; // clear flags
     int npart = getData->data.part;
     int presetNum = getData->data.engine;
     int min = 0;
@@ -408,7 +422,13 @@ float Dynamlimit::getlimits(CommandBlock *getData)
             break;
         case 9:
             break;
-        case 16:
+        case EFFECT::control::bpm:
+            max = 1;
+            canLearn = 0;
+            break;
+        case EFFECT::control::bpmStart:
+            break;
+        case EFFECT::control::preset:
             max = 4;
             canLearn = 0;
             break;
@@ -421,9 +441,9 @@ float Dynamlimit::getlimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
             break;
         case TOPLEVEL::type::Minimum:
