@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010, Alan Calvert
-    Copyright 2018-2019, Will Godfrey
+    Copyright 2018-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -24,32 +24,25 @@
 
 */
 
+#include "Misc/NumericFuncs.h"
 #include "Misc/SynthEngine.h"
 #include "Effects/EQ.h"
-#include "Misc/NumericFuncs.h"
 
-using func::rap2dB;
+using func::power;
+using func::powFrac;
+using func::asDecibel;
 
 
-EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
-    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0),
-    synth(_synth)
+EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth)
+    : Effect(insertion_, efxoutl_, efxoutr_, NULL, 0, _synth)
+    , Pchanged{false}
+    , Pvolume{}
+    , Pband{0}
+    , filter{*synth,*synth,*synth,*synth,*synth,*synth,*synth,*synth} // MAX_EQ_BANDS
 {
-    for (int i = 0; i < MAX_EQ_BANDS; ++i)
-    {
-        filter[i].Ptype = 0;
-        filter[i].Pfreq = 64;
-        filter[i].Pgain = 64;
-        filter[i].Pq = 64;
-        filter[i].Pstages = 0;
-        filter[i].l = new AnalogFilter(6, 1000.0, 1.0, 0, synth);
-        filter[i].r = new AnalogFilter(6, 1000.0, 1.0, 0, synth);
-    }
     // default values
     setvolume(50);
-    Pband = 0;
     setpreset(Ppreset);
-    Pchanged = false;
     cleanup();
 }
 
@@ -57,6 +50,7 @@ EQ::EQ(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth) :
 // Cleanup the effect
 void EQ::cleanup(void)
 {
+    Effect::cleanup();
     for (int i = 0; i < MAX_EQ_BANDS; ++i)
     {
         filter[i].l->cleanup();
@@ -68,6 +62,8 @@ void EQ::cleanup(void)
 // Effect output
 void EQ::out(float *smpsl, float *smpsr)
 {
+    outvolume.advanceValue(synth->sent_buffersize);
+
     memcpy(efxoutl, smpsl, synth->sent_bufferbytes);
     memcpy(efxoutr, smpsr, synth->sent_bufferbytes);
     for (int i = 0; i < synth->sent_buffersize; ++i)
@@ -122,7 +118,7 @@ void EQ::out(float *smpsl, float *smpsr)
 void EQ::setvolume(unsigned char Pvolume_)
 {
     Pvolume = Pvolume_;
-    float tmp = powf(0.005f, (1.0f - Pvolume / 127.0f)) * 10.0f;
+    float tmp = 10.0f * powFrac<200>(1.0f - Pvolume / 127.0f);
     outvolume.setTargetValue(tmp);
     volume.setTargetValue((!insertion) ? 1.0f : tmp);
 }
@@ -160,6 +156,7 @@ void EQ::changepar(int npar, unsigned char value)
             break;
         case 1:
             Pband = value;
+            break;
     }
     if (npar < 10)
         return;
@@ -185,7 +182,7 @@ void EQ::changepar(int npar, unsigned char value)
 
         case 1:
             filter[nb].Pfreq = value;
-            tmp = 600.0f * powf(30.0f, (value - 64.0f) / 64.0f);
+            tmp = 600.0f * power<30>((value - 64.0f) / 64.0f);
             filter[nb].freq.setTargetValue(tmp);
             break;
 
@@ -197,7 +194,7 @@ void EQ::changepar(int npar, unsigned char value)
 
         case 3:
             filter[nb].Pq = value;
-            tmp = powf(30.0f, (value - 64.0f) / 64.0f);
+            tmp = power<30>((value - 64.0f) / 64.0f);
             filter[nb].q.setTargetValue(tmp);
             break;
 
@@ -267,15 +264,15 @@ float EQ::getfreqresponse(float freq)
         resp *= filter[i].l->H(freq);
     }
     // Only for UI purposes, use target value.
-    return rap2dB(resp * outvolume.getTargetValue());
+    return asDecibel(resp * outvolume.getTargetValue());
 }
 
 
 float EQlimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value.F;
+    int value = getData->data.value;
     int control = getData->data.control;
-    int request = getData->data.type & 3; // clear upper bits
+    int request = getData->data.type & TOPLEVEL::type::Default; // clear flags
 
     int min = 0;
     int max = 127;
@@ -314,9 +311,9 @@ float EQlimit::getlimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
             break;
         case TOPLEVEL::type::Minimum:
