@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2018-2019, Will Godfrey
+    Copyright 2018-2021, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -24,10 +24,15 @@
 
 */
 
+#include "Misc/NumericFuncs.h"
+#include "Misc/SynthHelper.h"
 #include "Misc/SynthEngine.h"
 #include "Effects/Echo.h"
 #include <iostream>
 
+using func::power;
+using func::powFrac;
+
 static const int PRESET_SIZE = 7;
 static const int NUM_PRESETS = 9;
 static unsigned char presets[NUM_PRESETS][PRESET_SIZE] = {
@@ -43,21 +48,29 @@ static unsigned char presets[NUM_PRESETS][PRESET_SIZE] = {
     };
 
 Echo::Echo(bool insertion_, float* efxoutl_, float* efxoutr_, SynthEngine *_synth) :
-    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0),
+    Effect(insertion_, efxoutl_, efxoutr_, NULL, 0, _synth),
     Pdelay(60),
     Plrdelay(100),
+    feedback(1, _synth->samplerate),
+    hidamp(1, _synth->samplerate),
     lrdelay(0),
     ldelay(NULL),
     rdelay(NULL),
-    synth(_synth)
+    lxfade(1, _synth->samplerate_f),
+    rxfade(1, _synth->samplerate_f)
 {
     setvolume(50);
-    setfb(40);
+    setfeedback(40);
     sethidamp(60);
     setpreset(Ppreset);
     changepar(4, 30); // lrcross
     Pchanged = false;
+    maxdelay = 5 * synth->samplerate;
+    ldelay = new float[maxdelay];
+    rdelay = new float[maxdelay];
+    realposl = realposr = 1;
     cleanup();
+    initdelays();
 }
 
 
@@ -71,8 +84,13 @@ Echo::~Echo()
 // Cleanup the effect
 void Echo::cleanup(void)
 {
-    memset(ldelay, 0, dl * sizeof(float));
-    memset(rdelay, 0, dr * sizeof(float));
+    Effect::cleanup();
+    feedback.pushToTarget();
+    hidamp.pushToTarget();
+    lxfade.pushToTarget();
+    rxfade.pushToTarget();
+    memset(ldelay, 0, maxdelay * sizeof(float));
+    memset(rdelay, 0, maxdelay * sizeof(float));
     oldl = oldr = 0.0f;
 }
 
@@ -80,22 +98,42 @@ void Echo::cleanup(void)
 // Initialize the delays
 void Echo::initdelays(void)
 {
-    // todo: make this adjust insted of destroy old delays
-    kl = kr = 0;
+    if (Pbpm)
+    {
+        int olddelay = delay;
+
+        std::pair<float, float> frac = func::LFOfreqBPMFraction((float)Pdelay / 127.0f);
+        delay = roundf(60.0f * synth->samplerate_f * frac.first / (synth->getBPM() * frac.second));
+        if (delay > maxdelay)
+            delay = maxdelay;
+
+        if (!synth->isBPMAccurate())
+        {
+            // If we don't have an accurate BPM source, we may have
+            // fluctuations. Reject delay changes less than a certain amount to
+            // prevent Echo artifacts.
+
+            float ratio;
+            if (delay > olddelay)
+                ratio = (float)delay / (float)olddelay;
+            else
+                ratio = (float)olddelay / (float)delay;
+            if (ratio < ECHO_INACCURATE_BPM_THRESHOLD)
+                delay = olddelay;
+        }
+    }
+    else
+    {
+        delay = int(Pdelay / 127.0f * synth->samplerate_f * 1.5f);
+        delay += 1; // 0 .. 1.5 sec
+    }
+
     dl = delay - lrdelay;
     if (dl < 1)
         dl = 1;
     dr = delay + lrdelay;
     if (dr < 1)
         dr = 1;
-
-    if (ldelay != NULL)
-        delete [] ldelay;
-    if (rdelay != NULL)
-        delete [] rdelay;
-    ldelay = new float[dl];
-    rdelay = new float[dr];
-    cleanup();
 }
 
 
@@ -103,12 +141,51 @@ void Echo::initdelays(void)
 void Echo::out(float* smpsl, float* smpsr)
 {
     float l, r;
-    float ldl = ldelay[kl];
-    float rdl = rdelay[kr];
+    float ldl;
+    float rdl;
+
+    outvolume.advanceValue(synth->sent_buffersize);
+
+    initdelays();
+
     for (int i = 0; i < synth->sent_buffersize; ++i)
     {
-        ldl = ldelay[kl];
-        rdl = rdelay[kr];
+        lxfade.setTargetValue(dl);
+        rxfade.setTargetValue(dr);
+
+        int targetpos;
+
+        targetpos = realposl - lxfade.getNewValue();
+        if (targetpos < 0)
+            targetpos += maxdelay;
+        ldl = ldelay[targetpos];
+
+        targetpos = realposr - rxfade.getNewValue();
+        if (targetpos < 0)
+            targetpos += maxdelay;
+        rdl = rdelay[targetpos];
+
+        if (lxfade.isInterpolating())
+        {
+            targetpos = realposl - lxfade.getOldValue();
+            if (targetpos < 0)
+                targetpos += maxdelay;
+
+            ldl = ldelay[targetpos] * (1.0f - lxfade.factor()) + ldl * lxfade.factor();
+        }
+
+        if (rxfade.isInterpolating())
+        {
+            targetpos = realposr - rxfade.getOldValue();
+            if (targetpos < 0)
+                targetpos += maxdelay;
+
+            rdl = rdelay[targetpos] * (1.0f - rxfade.factor()) + rdl * rxfade.factor();
+        }
+
+        ldl += float(1e-20); // anti-denormal included
+        rdl += float(1e-20); // anti-denormal included
+
         l = ldl * (1.0 - lrcross.getValue()) + rdl * lrcross.getValue();
         r = rdl * (1.0 - lrcross.getValue()) + ldl * lrcross.getValue();
         lrcross.advanceValue();
@@ -118,21 +195,24 @@ void Echo::out(float* smpsl, float* smpsr)
         efxoutl[i] = ldl * 2.0f - 1e-20f; // anti-denormal - a very, very, very
         efxoutr[i] = rdl * 2.0f - 1e-20f; // small dc bias
 
-        ldl = smpsl[i] * pangainL.getAndAdvanceValue() - ldl * fb.getValue();
-        rdl = smpsr[i] * pangainR.getAndAdvanceValue() - rdl * fb.getValue();
-        fb.advanceValue();
+        ldl = smpsl[i] * pangainL.getAndAdvanceValue() - ldl * feedback.getValue();
+        rdl = smpsr[i] * pangainR.getAndAdvanceValue() - rdl * feedback.getValue();
+        feedback.advanceValue();
 
         // LowPass Filter
-        ldelay[kl] = ldl = ldl * hidamp.getValue() + oldl * (1.0f - hidamp.getValue());
-        rdelay[kr] = rdl = rdl * hidamp.getValue() + oldr * (1.0f - hidamp.getValue());
+        ldelay[realposl] = ldl = ldl * hidamp.getValue() + oldl * (1.0f - hidamp.getValue());
+        rdelay[realposl] = rdl = rdl * hidamp.getValue() + oldr * (1.0f - hidamp.getValue());
         hidamp.advanceValue();
         oldl = ldl;
         oldr = rdl;
 
-        if (++kl >= dl)
-            kl = 0;
-        if (++kr >= dr)
-            kr = 0;
+        if (++realposl >= maxdelay)
+            realposl = 0;
+        if (++realposr >= maxdelay)
+            realposr = 0;
+
+        lxfade.advanceValue();
+        rxfade.advanceValue();
     }
 }
 
@@ -143,7 +223,7 @@ void Echo::setvolume(unsigned char Pvolume_)
     Pvolume = Pvolume_;
     if (insertion == 0)
     {
-        outvolume.setTargetValue(powf(0.01f, (1.0f - Pvolume / 127.0f)) * 4.0f);
+        outvolume.setTargetValue(4.0f * powFrac<100>(1.0f - Pvolume / 127.0f));
         volume.setTargetValue(1.0f);
     }
     else
@@ -160,9 +240,6 @@ void Echo::setvolume(unsigned char Pvolume_)
 void Echo::setdelay(const unsigned char Pdelay_)
 {
     Pdelay = Pdelay_;
-    delay = int(Pdelay / 127.0f * synth->samplerate_f * 1.5f);
-    delay += 1; // 0 .. 1.5 sec
-    initdelays();
 }
 
 
@@ -170,18 +247,17 @@ void Echo::setlrdelay(unsigned char Plrdelay_)
 {
     float tmp;
     Plrdelay = Plrdelay_;
-    tmp = (powf(2.0f, fabsf(Plrdelay - 64.0f) / 64.0f * 9.0f) -1.0f) / 1000.0f * synth->samplerate_f;
+    tmp = (power<2>(fabsf(Plrdelay - 64.0f) / 64.0f * 9.0f) -1.0f) / 1000.0f * synth->samplerate_f;
     if (Plrdelay < 64.0f)
         tmp = -tmp;
     lrdelay = (int)tmp;
-    initdelays();
 }
 
 
-void Echo::setfb(unsigned char Pfb_)
+void Echo::setfeedback(unsigned char Pfb_)
 {
     Pfb = Pfb_;
-    fb.setTargetValue(Pfb / 128.0f);
+    feedback.setTargetValue(Pfb / 128.0f);
 }
 
 
@@ -202,6 +278,8 @@ void Echo::setpreset(unsigned char npreset)
             changepar(n, presets[npreset][n]);
         if (insertion)
             changepar(0, presets[npreset][0] / 2); // lower the volume if this is insertion effect
+        // All presets use no BPM syncing.
+        changepar(EFFECT::control::bpm, 0);
         Ppreset = npreset;
     }
     else
@@ -249,14 +327,20 @@ void Echo::changepar(int npar, unsigned char value)
             break;
 
         case 5:
-            setfb(value);
+            setfeedback(value);
             break;
 
         case 6:
             sethidamp(value);
             break;
+
+        case EFFECT::control::bpm:
+            Pbpm = value;
+            break;
+
         default:
             Pchanged = false;
+            break;
     }
 }
 
@@ -273,6 +357,7 @@ unsigned char Echo::getpar(int npar)
         case 4: return Plrcross;
         case 5: return Pfb;
         case 6: return Phidamp;
+        case EFFECT::control::bpm: return Pbpm;
         default: break;
     }
     return 0; // in case of bogus parameter number
@@ -281,9 +366,9 @@ unsigned char Echo::getpar(int npar)
 
 float Echolimit::getlimits(CommandBlock *getData)
 {
-    int value = getData->data.value.F;
+    int value = getData->data.value;
     int control = getData->data.control;
-    int request = getData->data.type & 3; // clear upper bits
+    int request = getData->data.type & TOPLEVEL::type::Default; // clear flags
     int npart = getData->data.part;
     int presetNum = getData->data.engine;
     int min = 0;
@@ -310,7 +395,11 @@ float Echolimit::getlimits(CommandBlock *getData)
             break;
         case 6:
             break;
-        case 16:
+        case EFFECT::control::bpm:
+            max = 1;
+            canLearn = 0;
+            break;
+        case EFFECT::control::preset:
             max = 8;
             canLearn = 0;
             break;
@@ -323,9 +412,9 @@ float Echolimit::getlimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
             break;
         case TOPLEVEL::type::Minimum:
