@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2019, Will Godfrey
+    Copyright 2017-2022, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -24,42 +24,31 @@
 
 */
 
-#include <fftw3.h>
 #include <iostream>
 
 #include "Misc/SynthEngine.h"
 #include "Effects/EffectMgr.h"
 
 EffectMgr::EffectMgr(const bool insertion_, SynthEngine *_synth) :
-    Presets(_synth),
-    insertion(insertion_),
-    filterpars(NULL),
-    nefx(0),
-    efx(NULL),
-    dryonly(false)
+    Presets{_synth},
+    efxoutl{size_t(_synth->buffersize)},
+    efxoutr{size_t(_synth->buffersize)},
+    insertion{insertion_},
+    filterpars{NULL},
+    nefx{0}, // type none resolves to zero internally
+    dryonly{false},
+    efx{}
 {
     setpresettype("Peffect");
-    efxoutl = (float*)fftwf_malloc(synth->bufferbytes);
-    efxoutr = (float*)fftwf_malloc(synth->bufferbytes);
-    memset(efxoutl, 0, synth->bufferbytes);
-    memset(efxoutr, 0, synth->bufferbytes);
-    InterpolatedParameter::setSampleRate(synth->samplerate_f);
     defaults();
 }
 
 
-EffectMgr::~EffectMgr()
-{
-    if (efx)
-        delete efx;
-    fftwf_free(efxoutl);
-    fftwf_free(efxoutr);
-}
 
 
 void EffectMgr::defaults(void)
 {
-    changeeffect(0);
+    changeeffect(0); // type none resolves to zero internally
     setdryonly(false);
 }
 
@@ -70,48 +59,45 @@ void EffectMgr::changeeffect(int _nefx)
     cleanup();
     if (nefx == _nefx)
         return;
+    //std::cout << "Change eff" << std::endl;
     nefx = _nefx;
-    memset(efxoutl, 0, synth->bufferbytes);
-    memset(efxoutr, 0, synth->bufferbytes);
-    if (efx)
-        delete efx;
-    switch (nefx)
+    switch (nefx + EFFECT::type::none)
     {
-        case 1:
-            efx = new Reverb(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::reverb:
+            efx.reset(new Reverb{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 2:
-            efx = new Echo(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::echo:
+            efx.reset(new Echo{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 3:
-            efx = new Chorus(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::chorus:
+            efx.reset(new Chorus{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 4:
-            efx = new Phaser(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::phaser:
+            efx.reset(new Phaser{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 5:
-            efx = new Alienwah(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::alienWah:
+            efx.reset(new Alienwah{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 6:
-            efx = new Distorsion(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::distortion:
+            efx.reset(new Distorsion{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 7:
-            efx = new EQ(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::eq:
+            efx.reset(new EQ{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
-        case 8:
-            efx = new DynamicFilter(insertion, efxoutl, efxoutr, synth);
+        case EFFECT::type::dynFilter:
+            efx.reset(new DynamicFilter{insertion, efxoutl.get(), efxoutr.get(), synth});
             break;
 
             // put more effect here
         default:
-            efx = NULL;
+            efx.reset();
             break; // no effect (thru)
     }
     if (efx)
@@ -129,6 +115,8 @@ int EffectMgr::geteffect(void)
 // Cleanup the current effect
 void EffectMgr::cleanup(void)
 {
+    memset(efxoutl.get(), 0, synth->bufferbytes);
+    memset(efxoutr.get(), 0, synth->bufferbytes);
     if (efx)
         efx->cleanup();
 }
@@ -185,19 +173,19 @@ void EffectMgr::out(float *smpsl, float *smpsr)
         {
             memset(smpsl, 0, synth->sent_bufferbytes);
             memset(smpsr, 0, synth->sent_bufferbytes);
-            memset(efxoutl, 0, synth->sent_bufferbytes);
-            memset(efxoutr, 0, synth->sent_bufferbytes);
+            memset(efxoutl.get(), 0, synth->sent_bufferbytes);
+            memset(efxoutr.get(), 0, synth->sent_bufferbytes);
         }
         return;
     }
-    memset(efxoutl, 0, synth->sent_bufferbytes);
-    memset(efxoutr, 0, synth->sent_bufferbytes);
+    memset(efxoutl.get(), 0, synth->sent_bufferbytes);
+    memset(efxoutr.get(), 0, synth->sent_bufferbytes);
     efx->out(smpsl, smpsr);
 
-    if (nefx == 7)
+    if (nefx == (EFFECT::type::eq - EFFECT::type::none))
     {   // this is need only for the EQ effect
-        memcpy(smpsl, efxoutl, synth->sent_bufferbytes);
-        memcpy(smpsr, efxoutr, synth->sent_bufferbytes);
+        memcpy(smpsl, efxoutl.get(), synth->sent_bufferbytes);
+        memcpy(smpsr, efxoutr.get(), synth->sent_bufferbytes);
         return;
     }
 
@@ -217,7 +205,7 @@ void EffectMgr::out(float *smpsl, float *smpsr)
                 v2 = 1.0f;
             }
             if (nefx == 1 || nefx==2)
-                v2 *= v2; // for Reverb and Echo, the wet function is not liniar
+                v2 *= v2; // for Reverb and Echo, the wet function is not linear
 
             if (dryonly)
             {
@@ -248,8 +236,7 @@ void EffectMgr::out(float *smpsl, float *smpsr)
 // Get the effect volume for the system effect
 float EffectMgr::sysefxgetvolume(void)
 {
-    // No interpolation for system effect currently (direct target value).
-    return (!efx) ? 1.0f : efx->outvolume.getTargetValue();
+    return (!efx) ? 1.0f : efx->outvolume.getValue();
 }
 
 
@@ -296,7 +283,7 @@ void EffectMgr::add2XML(XMLwrapper *xml)
 
 void EffectMgr::getfromXML(XMLwrapper *xml)
 {
-    changeeffect(xml->getpar127("type", geteffect()));
+    changeeffect(xml->getpar127("type", geteffect())); // not convinced this is OK?
     if (!efx || !geteffect())
         return;
     changepreset(xml->getpar127("preset", efx->Ppreset));
@@ -329,7 +316,7 @@ void EffectMgr::getfromXML(XMLwrapper *xml)
             }
         }
         xml->exitbranch();
-        //if(geteffectpar(-1))
+        //if (geteffectpar(-1))
             //cout << "Some pars changed" << endl;
     }
     cleanup();
@@ -379,6 +366,7 @@ float LimitMgr::geteffectlimits(CommandBlock *getData)
             break;
         default:
             value = EFFECT::type::count - EFFECT::type::none;
+            break;
     }
     return value;
 }
