@@ -7,7 +7,7 @@
     Copyright 2018-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -28,11 +28,13 @@
 #ifndef REVERB_H
 #define REVERB_H
 
+#include "Misc/Alloc.h"
 #include "Effects/Effect.h"
 
 #define REV_COMBS 8
 #define REV_APS 4
 
+
 class Unison;
 class AnalogFilter;
 
@@ -43,7 +45,7 @@ class Reverb : public Effect
     public:
         Reverb(bool insertion_, float *efxoutl_, float *efxoutr_, SynthEngine *_synth);
         ~Reverb();
-        void out(float *smps_l, float *smps_r);
+        void out(float *rawL, float *rawR);
         void cleanup(void);
 
         void setpreset(unsigned char npreset);
@@ -52,14 +54,16 @@ class Reverb : public Effect
 
 
     private:
+        static constexpr size_t NUM_TYPES = 3;
+
         // Parametrii
         bool Pchanged;
         unsigned char Pvolume;
         unsigned char Ptime;
         unsigned char Pidelay;
         unsigned char Pidelayfb;
-        unsigned char Prdelay;
-        unsigned char Perbalance;
+//        unsigned char Prdelay;  // **** RHL ****
+//        unsigned char Perbalance;   // **** RHL ****
         unsigned char Plpf;
         unsigned char Phpf; // todo 0..63 lpf, 64 = off, 65..127 = hpf(TODO)
         unsigned char Plohidamp;
@@ -75,41 +79,43 @@ class Reverb : public Effect
         void setidelayfb(unsigned char Pidelayfb_);
         void sethpf(unsigned char Phpf_);
         void setlpf(unsigned char Plpf_);
-        void settype( unsigned char Ptype_);
+        void settype(unsigned char Ptype_);
         void setroomsize(unsigned char Proomsize_);
         void setbandwidth(unsigned char Pbandwidth_);
-        void processmono(int ch, float *output);
 
-        float erbalance;
+//        float erbalance;    // **** RHL ****
 
         // Parametrii 2
         int lohidamptype; // 0 = disable, 1 = highdamp (lowpass), 2 = lowdamp (highpass)
         int idelaylen;
-        int rdelaylen;
+//        int rdelaylen;  // **** RHL ****
         int idelayk;
         float lohifb;
         float idelayfb;
         float roomsize;
         float rs; // rs is used to "normalise" the volume according to the roomsize
-        int comblen[REV_COMBS * 2];
-        int aplen[REV_APS * 2];
+        size_t comblen[REV_COMBS * 2];   // length for each CombFilter feedback line (random)
+        size_t aplen[REV_APS * 2];       // length for each AllPass feedback line (random)
         Unison *bandwidth;
 
         // Internal Variables
-        float *comb[REV_COMBS * 2];
-        int combk[REV_COMBS * 2];
-        float combfb[REV_COMBS * 2];// <feedback-ul fiecarui filtru "comb"
-        float lpcomb[REV_COMBS * 2];  // <pentru Filtrul LowPass
-        float *ap[REV_APS * 2];
-        int apk[REV_APS * 2];
-        float *idelay;
-        AnalogFilter *lpf;  // filters
-        AnalogFilter *hpf;
-        InterpolatedParameter lpffr;
-        InterpolatedParameter hpffr;
-        float *inputbuf;
+        float *comb[REV_COMBS * 2];   // N CombFilter pipelines for each channel
+        size_t combk[REV_COMBS * 2];  // current offset of the comb insertion point (cycling)
+        float combfb[REV_COMBS * 2];  // feedback coefficient of each Comb-filter
+        float lpcomb[REV_COMBS * 2];  // LowPass filtered output feedback from Comb
+        float *ap[REV_APS * 2];       // AllPass-filter
+        size_t apk[REV_APS * 2];      // current offset of the AllPass insertion point (cycling)
+        float *idelay;                // Input delay line
+        AnalogFilter *lpf;            // LowPass-filter on the input
+        AnalogFilter *hpf;            // HighPass-filter on the input
+        synth::InterpolatedValue<float> lpffr;
+        synth::InterpolatedValue<float> hpffr;
+        Samples inputbuf;
 
-        SynthEngine *synth;
+        void preprocessInput(float *rawL, float *rawR, Samples& inputFeed);
+        void calculateReverb(size_t ch, Samples& inputFeed, float *output);
+        void setupPipelines();
+        void clearBuffers();
 };
 
 class Revlimit
