@@ -281,16 +281,6 @@ void YoshimiLV2Plugin::processMidiMessage(const uint8_t * msg)
 }
 
 
-void *YoshimiLV2Plugin::idleThread()
-{
-    while (_synth->getRuntime().runSynth)
-    {
-        usleep(33333); // TODO what is the point of this?
-    }
-    return NULL;
-}
-
-
 YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const char *bundlePath, const LV2_Feature *const *features, const LV2_Descriptor *desc):
     MusicIO(synth, new SinglethreadedBeatTracker),
     _synth(synth),
@@ -303,7 +293,6 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
     _bufferPos(0),
     _offsetPos(0),
     _bFreeWheel(NULL),
-    _pIdleThread(0),
     _lv2_desc(desc)
 {
     _uridMap.handle = NULL;
@@ -364,7 +353,7 @@ YoshimiLV2Plugin::YoshimiLV2Plugin(SynthEngine *synth, double sampleRate, const
         }
     }
 
-    //_synth->getRuntime().Log("Buffer size " + to_string(nomBufSize));;
+    //_synth->getRuntime().Log("Buffer size " + to_string(nomBufSize));
     if (nomBufSize > 0)
         _bufferSize = nomBufSize;
     else if (_bufferSize == 0)
@@ -383,8 +372,6 @@ YoshimiLV2Plugin::~YoshimiLV2Plugin()
             getProgram(flatbankprgs.size() + 1);
         }
         _synth->getRuntime().runSynth = false;
-        if(_pIdleThread)
-            pthread_join(_pIdleThread, NULL);
         delete _synth;
         _synth = NULL;
     }
@@ -418,12 +405,6 @@ bool YoshimiLV2Plugin::init()
 
     _synth->getRuntime().runSynth = true;
 
-    if (!_synth->getRuntime().startThread(&_pIdleThread, YoshimiLV2Plugin::static_idleThread, this, false, 0, "LV2 idle"))
-    {
-        synth->getRuntime().Log("Failed to start idle thread");
-        return false;
-    }
-
     synth->getRuntime().Log("Starting in LV2 plugin mode");
     return true;
 }
@@ -667,12 +648,6 @@ void YoshimiLV2Plugin::selectProgramNew(unsigned char channel, uint32_t bank, ui
 }
 
 
-void *YoshimiLV2Plugin::static_idleThread(void *arg)
-{
-    return static_cast<YoshimiLV2Plugin *>(arg)->idleThread();
-}
-
-
 LV2_State_Status YoshimiLV2Plugin::static_StateSave(LV2_Handle instance, LV2_State_Store_Function store, LV2_State_Handle handle, uint32_t flags, const LV2_Feature * const *features)
 {
     return static_cast<YoshimiLV2Plugin *>(instance)->stateSave(store, handle, flags, features);
