@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2021, Will Godfrey & others
+    Copyright 2014-2022, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU General Public
@@ -62,9 +62,8 @@ class Config
         void Log(const string& msg, char tostderr = _SYS_::LogNormal);
     void LogError(const string& msg);
         void flushLog(void);
-
-        void clearPresetsDirlist(void);
-
+        bool loadPresetsList(void);
+        bool savePresetsList(void);
         bool saveConfig(bool master = false);
         bool loadConfig(void);
         void restoreConfig(SynthEngine *_synth);
@@ -184,7 +183,7 @@ class Config
         unsigned int  currentRoot;
         bool          bankHighlight;
         int           lastBankPart;
-        int           currentPreset;
+        int           presetsRootID;
         int           tempBank;
         int           tempRoot;
         int           noteOnSent; // note test
