@@ -2,7 +2,7 @@
     FormatFuncs.h
 
     Copyright 2010, Alan Calvert
-    Copyright 2014-2020, Will Godfrey and others.
+    Copyright 2014-2022, Will Godfrey and others.
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -26,11 +26,10 @@
 #include <string>
 #include <sstream>
 #include <cstring>
-
+#include <list>
 
 namespace func {
 
-
 inline std::string asString(int n)
 {
    std::ostringstream oss;
@@ -114,6 +113,17 @@ inline std::string asLongString(float n)
 }
 
 
+inline std::string asCompactString(float n)
+{
+   std::ostringstream oss;
+   oss.setf(std::ios_base::fixed, std::ios_base::floatfield);
+   oss.precision(1);
+   oss.width(1);
+   oss << n;
+   return oss.str();
+}
+
+
 inline std::string asHexString(int x)
 {
    std::ostringstream oss;
@@ -136,6 +146,17 @@ inline std::string asHexString(unsigned int x)
 }
 
 
+inline std::string asMidiNoteString(unsigned char n)
+{
+    static std::string note[] = {
+        "C","C#","D","D#","E","F","F#","G","G#","A","B","B#"
+    };
+    int octave = -1 + n/12;
+    int key   = n % 12;
+    return "("+note[key]+asString(octave)+")";
+}
+
+
 
 inline float string2float(std::string str)
 {
@@ -155,6 +176,16 @@ inline double string2double(std::string str)
 }
 
 
+inline bool isDigits(std::string str)
+{
+    if (str.empty())
+        return false;
+    char c = str.at(0);
+    if (c < '0' or c > '9')
+        return false;
+    return true;
+}
+
 inline int string2int(std::string str)
 {
     std::istringstream machine(str);
@@ -163,6 +194,14 @@ inline int string2int(std::string str)
     return intval;
 }
 
+inline int string2long(std::string str)
+{
+    std::istringstream machine(str);
+    long longval;
+    machine >> longval;
+    return longval;
+}
+
 
 /* ensures MIDI compatible numbers without errors */
 inline int string2int127(std::string str)
@@ -186,8 +225,11 @@ inline unsigned int string2uint(std::string str)
     return intval;
 }
 
-
-inline string stringCaps(std::string str, int count)
+/*
+ * turns the 1st count mumber to upper case
+ * all the rest to lower case
+ */
+inline std::string stringCaps(std::string str, int count)
 {
     int idx = 0;
     char c;
@@ -229,33 +271,62 @@ inline int findSplitPoint(std::string name)
  */
 inline std::string formatTextLines(std::string text, size_t maxLen)
 {
-    size_t lineLen = text.length();
-    if (lineLen < maxLen)
+    size_t totalLen = text.length();
+    if (totalLen < maxLen)
         return text;
-    const char lineEnd = char(10);
-    size_t pos = maxLen;
-    size_t oldPos = 0;
-
-    while (pos < lineLen && pos > oldPos)
+    size_t pos = 0;
+    size_t ref = 0;
+    while (pos < totalLen) // split overlong words first
     {
-        pos = text.rfind(' ', pos);
-        if(pos < oldPos)
+        if (text.at(pos) < '!')
         {
-            pos = oldPos + maxLen + 1; // inserted char
-            text.insert(pos, 1, lineEnd);
+            ++ pos;
+            ref = pos;
         }
-        else
-            text.replace(pos, 1, 1, lineEnd);
-        oldPos = pos;
-        pos += maxLen + 1; // replaced char
-        lineLen += 1;
+        if ((pos - ref) > maxLen)
+        {
+            text.insert(pos, 1, '\n');
+            ++ totalLen;
+            ++ pos;
+            ref = pos;
+        }
+        ++pos;
     }
+
+    pos = 0;
+    ref = 0;
+    size_t lastSpace = 0;
+    while (pos < text.length())
+    {
+        if (text.at(pos) == '\n') // skip over existing line ends
+        {
+            ++ pos;
+            ref = pos;
+            lastSpace = 0;
+        }
+        else if (text.at(pos) == ' ')
+            lastSpace = pos;
+        if ((pos - ref) >= maxLen)
+        {
+            if (lastSpace == 0)
+                pos = text.length();
+            else
+            {
+                text.at(lastSpace)= '\n';
+                ref = lastSpace;
+                lastSpace = 0;
+            }
+        }
+        ++ pos;
+    }
+    while (text.at(text.length() - 1) < '!') // tidy up
+        text.pop_back();
     return text;
 }
 
 
 inline std::string nextLine(std::string& list) // this is destructive
-{
+{ // currently only used in main
     size_t pos = list.find('\n');
     std::string line = "";
     if (pos == std::string::npos)
