@@ -7,7 +7,7 @@
     Copyright 2016-2019, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -31,11 +31,13 @@
 #include <cmath>
 #include <string>
 #include "globals.h"
+#include "Misc/NumericFuncs.h"
 
 class SynthEngine;
 class XMLwrapper;
 
 using std::string;
+using func::power;
 
 const size_t MAX_OCTAVE_SIZE = 128;
 
@@ -77,8 +79,8 @@ class Microtonal
         void tuningtoline(unsigned int n, char *line, int maxn);
         string tuningtotext(void);
         string keymaptotext(void);
-        int loadscl(string filename); // load the tunings from a .scl file
-        int loadkbm(string filename); // load the mapping from .kbm file
+        int loadscl(const string& filename); // load the tunings from a .scl file
+        int loadkbm(const string& filename); // load the mapping from .kbm file
         int texttotunings(const char *text);
         int texttomapping(const char *text);
 
@@ -87,14 +89,14 @@ class Microtonal
 
         void add2XML(XMLwrapper *xml);
         void getfromXML(XMLwrapper *xml);
-        bool saveXML(string filename);
-        bool loadXML(string filename);
+        bool saveXML(const string& filename);
+        bool loadXML(const string& filename);
 
     private:
         string reformatline(string text);
         bool validline(const char *line);
         int linetotunings(unsigned int nline, const char *line);
-        int loadLine(string text, size_t &point, char *line, size_t maxlen);
+        int loadLine(const string& text, size_t &point, char *line, size_t maxlen);
         // loads a line from the text file,
         // ignoring the lines beginning with "!"
         size_t octavesize;
@@ -110,8 +112,6 @@ class Microtonal
         Octave octave[MAX_OCTAVE_SIZE];
         Octave tmpoctave[MAX_OCTAVE_SIZE];
 
-        float note_12et[128];
-
         SynthEngine *synth;
 };
 
@@ -122,7 +122,7 @@ inline int Microtonal::getoctavesize(void)
 
 inline float Microtonal::getFixedNoteFreq(int note)
 {
-    return powf(2.0f, (float)(note - PrefNote) / 12.0f) * PrefFreq;
+    return power<2>(float(note - PrefNote) / 12.0f) * PrefFreq;
 }
 
 
