@@ -2,7 +2,7 @@
     AlsaEngine.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2019, Will Godfrey & others
+    Copyright 2014-2021, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -29,8 +29,12 @@
 
 using func::asString;
 
+// The number of nanoseconds before the MIDI clock is assumed missing.
+#define MIDI_CLOCK_TIMEOUT_US 1000000
 
-AlsaEngine::AlsaEngine(SynthEngine *_synth) :MusicIO(_synth)
+
+AlsaEngine::AlsaEngine(SynthEngine *_synth, BeatTracker *_beatTracker) :
+    MusicIO(_synth, _beatTracker)
 {
     audio.handle = NULL;
     audio.period_count = 0; // re-used as number of periods
@@ -42,8 +46,22 @@ AlsaEngine::AlsaEngine(SynthEngine *_synth) :MusicIO(_synth)
     audio.pThread = 0;
 
     midi.handle = NULL;
+    midi.addr.client = 0;
+    midi.addr.port = 0;
     midi.alsaId = -1;
     midi.pThread = 0;
+    midi.lastDivSongBeat = 0;
+    midi.lastDivMonotonicBeat = 0;
+    midi.clockCount = 0;
+
+    for (int i = 0; i < ALSA_MIDI_BPM_MEDIAN_WINDOW; i++)
+        midi.prevBpms[i] = 120;
+    midi.prevBpmsPos = 0;
+
+    timespec time;
+    clock_gettime(CLOCK_MONOTONIC, &time);
+    midi.prevClockUs = time.tv_sec * 1000000 + time.tv_nsec / 1000;
+
     little_endian = synth->getRuntime().isLittleEndian;
 }
 
@@ -66,14 +84,8 @@ bool AlsaEngine::openAudio(void)
         {
             if (prepSwparams())
             {
-                if (prepBuffers())
-                {
-                    int buffersize = getBuffersize();
-                    interleaved = new int[buffersize * card_chans];
-                    if (NULL == interleaved)
-                        goto bail_out;
-                    memset(interleaved, 0, sizeof(int) * buffersize * card_chans);
-                }
+                prepBuffers();
+                interleaved.reset(new int[getBuffersize() * card_chans]{0});
             }
         }
     }
@@ -119,6 +131,8 @@ std::string AlsaEngine::findMidiClients(snd_seq_t *seq)
 
 bool AlsaEngine::openMidi(void)
 {
+    synth->setBPMAccurate(false);
+
     const char* port_name = "input";
     int port_num;
     if (snd_seq_open(&midi.handle, "default", SND_SEQ_OPEN_INPUT, SND_SEQ_NONBLOCK) != 0)
@@ -142,6 +156,8 @@ bool AlsaEngine::openMidi(void)
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_NONREGPARAM);
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_REGPARAM);
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_RESET);
+    snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_SONGPOS);
+    snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_CLOCK);
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_PORT_SUBSCRIBED);
     snd_seq_client_info_event_filter_add(seq_info, SND_SEQ_EVENT_PORT_UNSUBSCRIBED);
     if (0 > snd_seq_set_client_info(midi.handle, seq_info))
@@ -174,7 +190,7 @@ bool AlsaEngine::openMidi(void)
             return true;
     }
     std::string found = "";
-    if (!midilist.empty() && midilist != "default")
+    if (midilist != "default")
     {
         while (!midilist.empty())
         {
@@ -218,19 +234,19 @@ bool AlsaEngine::openMidi(void)
 
 void AlsaEngine::Close(void)
 {
-    if(synth->getRuntime().runSynth)
+    if (synth->getRuntime().runSynth)
     {
         synth->getRuntime().runSynth = false;
     }
 
-    if(midi.pThread != 0) //wait for midi thread to finish
+    if (midi.pThread != 0) //wait for midi thread to finish
     {
         void *ret = NULL;
         pthread_join(midi.pThread, &ret);
         midi.pThread = 0;
     }
 
-    if(audio.pThread != 0) //wait for audio thread to finish
+    if (audio.pThread != 0) //wait for audio thread to finish
     {
         void *ret = NULL;
         pthread_join(audio.pThread, &ret);
@@ -263,7 +279,7 @@ std::string AlsaEngine::midiClientName(void)
     //Andrew Deryabin: for multi-instance support add unique id to
     //instances other then default (0)
     unsigned int synthUniqueId = synth->getUniqueId();
-    if(synthUniqueId > 0)
+    if (synthUniqueId > 0)
     {
         char sUniqueId [256];
         memset(sUniqueId, 0, sizeof(sUniqueId));
@@ -344,7 +360,7 @@ bool AlsaEngine::prepHwparams(void)
     else
         formattxt += "Big";
 
-    synth->getRuntime().Log("March is " + formattxt + " Endian", 2);
+    synth->getRuntime().Log("March is " + formattxt + " Endian", _SYS_::LogNotSerious);
 
     if (card_signed)
         formattxt = "Signed ";
@@ -387,7 +403,7 @@ bool AlsaEngine::prepHwparams(void)
     if (ask_buffersize != audio.period_size)
     {
         synth->getRuntime().Log("Asked for buffersize " + asString(ask_buffersize, 2)
-                    + ", Alsa dictates " + asString((unsigned int)audio.period_size), 2);
+                    + ", Alsa dictates " + asString((unsigned int)audio.period_size), _SYS_::LogNotSerious);
         synth->getRuntime().Buffersize = audio.period_size; // we shouldn't need to do this :(
     }
     return true;
@@ -430,10 +446,10 @@ bail_out:
 
 void AlsaEngine::Interleave(int buffersize)
 {
-    int idx = 0;
+    size_t idx = 0;
     bool byte_swap = (little_endian != card_endian);
     unsigned short int tmp16a, tmp16b;
-    int chans;
+    size_t chans;
     unsigned int tmp32a, tmp32b;
     unsigned int shift = 0x78000000;
     if (card_bits == 24)
@@ -488,6 +504,9 @@ void *AlsaEngine::AudioThread(void)
     alsaBad(snd_pcm_start(audio.handle), "alsa audio pcm start failed");
     while (synth->getRuntime().runSynth)
     {
+        BeatTracker::BeatValues beats(beatTracker->getBeatValues());
+        synth->setBeatValues(beats.songBeat, beats.monotonicBeat, beats.bpm);
+
         audio.pcm_state = snd_pcm_state(audio.handle);
         if (audio.pcm_state != SND_PCM_STATE_RUNNING)
         {
@@ -532,7 +551,7 @@ void *AlsaEngine::AudioThread(void)
 void AlsaEngine::Write(snd_pcm_uframes_t towrite)
 {
     snd_pcm_sframes_t wrote = 0;
-    int *data = interleaved;
+    int *data = interleaved.get();
 
     while (towrite > 0)
     {
@@ -654,107 +673,174 @@ void *AlsaEngine::_MidiThread(void *arg)
  */
 
 void *AlsaEngine::MidiThread(void)
+{
+    unsigned int pollCount = snd_seq_poll_descriptors_count(midi.handle, POLLIN);
+    struct pollfd pollfds[pollCount];
+
+    while (synth->getRuntime().runSynth)
+    {
+        snd_seq_poll_descriptors(midi.handle, pollfds, pollCount, POLLIN);
+
+        // Poll with timeout. Should be long-ish for performance reasons, but
+        // should be short enough to be smaller than MIDI_CLOCK_TIMEOUT_US, and
+        // short enough to be able to quit relatively quickly.
+        int pollResult = poll(pollfds, pollCount, 500);
+
+        if (pollResult < 0) {
+            if (errno == EINTR)
+                continue;
+            else {
+                char errMsg[200];
+                snprintf(errMsg, sizeof(errMsg),
+                    "Unable to handle error in MIDI thread: %s. Shutting down MIDI.",
+                    strerror(errno));
+                synth->getRuntime().Log(errMsg);
+                break;
+            }
+        }
+
+        timespec time;
+        clock_gettime(CLOCK_MONOTONIC, &time);
+        uint64_t clock = time.tv_sec * 1000000 + time.tv_nsec / 1000;
+
+        if (pollResult > 0)
+            handleMidiEvents(clock);
+
+        if ((clock - midi.prevClockUs) >= MIDI_CLOCK_TIMEOUT_US)
+            handleMidiClockSilence(clock);
+    }
+
+    return NULL;
+}
+
+void AlsaEngine::handleMidiEvents(uint64_t clock)
 {
     snd_seq_event_t *event;
     unsigned int par;
     int chk;
     bool sendit;
     unsigned char par0, par1 = 0, par2 = 0;
-    while (synth->getRuntime().runSynth)
+
+    while ((chk = snd_seq_event_input(midi.handle, &event)) > 0)
     {
-        while ((chk = snd_seq_event_input(midi.handle, &event)) > 0)
+        if (!event)
+            continue;
+        sendit = true;
+        par0 = event->data.control.channel;
+        par = 0;
+        switch (event->type)
         {
-            if (!event)
-                continue;
-            sendit = true;
-            par0 = event->data.control.channel;
-            par = 0;
-            switch (event->type)
-            {
-                case SND_SEQ_EVENT_NOTEON:
-                    par0 = event->data.note.channel;
-                    par0 |= 0x90;
-                    par1 = event->data.note.note;
-                    par2 = event->data.note.velocity;
-                    break;
+            case SND_SEQ_EVENT_NOTEON:
+                par0 = event->data.note.channel;
+                par0 |= 0x90;
+                par1 = event->data.note.note;
+                par2 = event->data.note.velocity;
+                break;
 
-                case SND_SEQ_EVENT_NOTEOFF:
-                    par0 = event->data.note.channel;
-                    par0 |= 0x80;
-                    par1 = event->data.note.note;
-                    break;
+            case SND_SEQ_EVENT_NOTEOFF:
+                par0 = event->data.note.channel;
+                par0 |= 0x80;
+                par1 = event->data.note.note;
+                break;
 
-                case SND_SEQ_EVENT_KEYPRESS:
-                    par0 = event->data.note.channel;
-                    par0 |= 0xa0;
-                    par1 = event->data.note.note;
-                    par2 = event->data.note.velocity;
-                    break;
+            case SND_SEQ_EVENT_KEYPRESS:
+                par0 = event->data.note.channel;
+                par0 |= 0xa0;
+                par1 = event->data.note.note;
+                par2 = event->data.note.velocity;
+                break;
 
-                case SND_SEQ_EVENT_CHANPRESS:
-                    par0 |= 0xd0;
-                    par1 = event->data.control.value;
-                    break;
+            case SND_SEQ_EVENT_CHANPRESS:
+                par0 |= 0xd0;
+                par1 = event->data.control.value;
+                break;
 
-                case SND_SEQ_EVENT_PGMCHANGE:
-                    par0 |= 0xc0;
-                    par1 = event->data.control.value;
-                    break;
+            case SND_SEQ_EVENT_PGMCHANGE:
+                par0 |= 0xc0;
+                par1 = event->data.control.value;
+                break;
 
-                case SND_SEQ_EVENT_PITCHBEND:
-                    par0 |= 0xe0;
-                    par = event->data.control.value + 8192;
-                    par1 = par & 0x7f;
-                    par2 = par >> 7;
-                    break;
+            case SND_SEQ_EVENT_PITCHBEND:
+                par0 |= 0xe0;
+                par = event->data.control.value + 8192;
+                par1 = par & 0x7f;
+                par2 = par >> 7;
+                break;
 
-                case SND_SEQ_EVENT_CONTROLLER:
-                    par0 |= 0xb0;
-                    par1 = event->data.control.param;
-                    par2 = event->data.control.value;
-                    break;
+            case SND_SEQ_EVENT_CONTROLLER:
+                par0 |= 0xb0;
+                par1 = event->data.control.param;
+                par2 = event->data.control.value;
+                break;
 
-                case SND_SEQ_EVENT_NONREGPARAM:
-                    par0 |= 0xb0; // splitting into separate CCs
-                    par = event->data.control.param;
-                    setMidi(par0, 99, par >> 7);
-                    setMidi(par0, 99, par & 0x7f);
-                    par = event->data.control.value;
-                    setMidi(par0, 6, par >> 7);
-                    par1 = 38;
-                    par2 = par & 0x7f; // let last one through
-                    break;
+            case SND_SEQ_EVENT_NONREGPARAM:
+                par0 |= 0xb0; // splitting into separate CCs
+                par = event->data.control.param;
+                setMidi(par0, 99, par >> 7);
+                setMidi(par0, 99, par & 0x7f);
+                par = event->data.control.value;
+                setMidi(par0, 6, par >> 7);
+                par1 = 38;
+                par2 = par & 0x7f; // let last one through
+                break;
 
-                case SND_SEQ_EVENT_RESET: // reset to power-on state
-                    par0 = 0xff;
-                    break;
+            case SND_SEQ_EVENT_RESET: // reset to power-on state
+                par0 = 0xff;
+                break;
 
-                case SND_SEQ_EVENT_PORT_SUBSCRIBED: // ports connected
-                    synth->getRuntime().Log("Alsa midi port connected");
-                    sendit = false;
-                    break;
+            case SND_SEQ_EVENT_PORT_SUBSCRIBED: // ports connected
+                synth->getRuntime().Log("Alsa midi port connected");
+                sendit = false;
+                break;
 
-                case SND_SEQ_EVENT_PORT_UNSUBSCRIBED: // ports disconnected
-                    synth->getRuntime().Log("Alsa midi port disconnected");
-                    sendit = false;
-                    break;
-                default:
-                    sendit = false;// commented out some progs spam us :(
-                    /* synth->getRuntime().Log("Other non-handled midi event, type: "
-                                + asString((int)event->type));*/
-                    break;
-            }
-            if (sendit)
-                setMidi(par0, par1, par2);
-            snd_seq_free_event(event);
-        }
-;
-        if(chk < 0)
-        {
-            usleep(1024);
+            case SND_SEQ_EVENT_PORT_UNSUBSCRIBED: // ports disconnected
+                synth->getRuntime().Log("Alsa midi port disconnected");
+                sendit = false;
+                break;
+
+            case SND_SEQ_EVENT_SONGPOS:
+                handleSongPos((float)event->data.control.value
+                    / (float)MIDI_SONGPOS_BEAT_DIVISION);
+                sendit = false;
+                break;
+
+            case SND_SEQ_EVENT_CLOCK:
+                handleMidiClock(clock);
+                sendit = false;
+                break;
+
+            default:
+                sendit = false;// commented out some progs spam us :(
+                /* synth->getRuntime().Log("Other non-handled midi event, type: "
+                            + asString((int)event->type));*/
+                break;
         }
+        if (sendit)
+            setMidi(par0, par1, par2);
+        snd_seq_free_event(event);
     }
-    return NULL;
+}
+
+void AlsaEngine::handleMidiClockSilence(uint64_t clock)
+{
+    // This is equivalent to receiving a clock beat every MIDI_CLOCK_TIMEOUT_US
+    // nanoseconds, except we do not use it to calculate the BPM, but use the
+    // fallback value instead. In between these fake "beats", the BeatTracker
+    // interpolates the values for us, as it also does for normal MIDI clock
+    // beats. This means it may take up to MIDI_CLOCK_TIMEOUT_US nanoseconds to
+    // react to a change in the BPM fallback.
+    BeatTracker::BeatValues beats {
+        midi.lastDivSongBeat,
+        midi.lastDivMonotonicBeat,
+        synth->PbpmFallback,
+    };
+    float diff = (clock - midi.prevClockUs) * beats.bpm / (60.0f * 1000000.0f);
+    beats.songBeat += diff;
+    beats.monotonicBeat += diff;
+    midi.lastDivSongBeat = beats.songBeat;
+    midi.lastDivMonotonicBeat = beats.monotonicBeat;
+    beatTracker->setBeatValues(beats);
+    midi.prevClockUs = clock;
 }
 
 
@@ -767,4 +853,82 @@ bool AlsaEngine::alsaBad(int op_result, std::string err_msg)
     return isbad;
 }
 
+void AlsaEngine::handleSongPos(float beat)
+{
+    const float subDiv = 1.0f / (float)(MIDI_CLOCKS_PER_BEAT / MIDI_CLOCK_DIVISION);
+
+    // The next MIDI clock should trigger this beat.
+    midi.lastDivSongBeat = beat - subDiv;
+
+    // Possibly adjust the monotonic beat backwards to avoid accumulating too
+    // many beats when we adjust clockCount below.
+    midi.lastDivMonotonicBeat -= (MIDI_CLOCK_DIVISION - midi.clockCount - 1) * subDiv;
+
+    // Force next clock tick to be a clean beat, on zero.
+    midi.clockCount = MIDI_CLOCK_DIVISION - 1;
+
+    // Tempting to call this here, but it is actually the next MIDI clock which
+    // signals the next beat.
+    //beatTracker->setBeatValues(beats);
+}
+
+void AlsaEngine::handleMidiClock(uint64_t clock)
+{
+    float bpm = 1000000.0f * 60.0f / float((clock - midi.prevClockUs) * MIDI_CLOCKS_PER_BEAT);
+    if (++midi.prevBpmsPos >= ALSA_MIDI_BPM_MEDIAN_WINDOW)
+        midi.prevBpmsPos = 0;
+    midi.prevBpms[midi.prevBpmsPos] = bpm;
+
+    float tmpBpms[ALSA_MIDI_BPM_MEDIAN_WINDOW];
+    memcpy(tmpBpms, midi.prevBpms+midi.prevBpmsPos, (ALSA_MIDI_BPM_MEDIAN_WINDOW - midi.prevBpmsPos)
+           * sizeof(*midi.prevBpms));
+    memcpy(tmpBpms + ALSA_MIDI_BPM_MEDIAN_WINDOW - midi.prevBpmsPos, midi.prevBpms, midi.prevBpmsPos
+           * sizeof(*midi.prevBpms));
+
+    // To avoid fluctuations in the BPM value due to clock inaccuracies, sort
+    // all the most recent bpm values, and take the average of the middle part
+    // (an average median). For this, we use Bubble sort, but we only need to
+    // sort the half that we use.
+    for (int i = 0; i < (ALSA_MIDI_BPM_MEDIAN_WINDOW+ALSA_MIDI_BPM_MEDIAN_AVERAGE_WINDOW)/2; i++)
+    {
+        for (int j = i+1; j < ALSA_MIDI_BPM_MEDIAN_WINDOW; j++)
+        {
+            if (tmpBpms[i] > tmpBpms[j])
+            {
+                float tmp = tmpBpms[i];
+                tmpBpms[i] = tmpBpms[j];
+                tmpBpms[j] = tmp;
+            }
+        }
+    }
+    bpm = 0;
+    for (int i = (ALSA_MIDI_BPM_MEDIAN_WINDOW - ALSA_MIDI_BPM_MEDIAN_AVERAGE_WINDOW) / 2;
+         i < (ALSA_MIDI_BPM_MEDIAN_WINDOW + ALSA_MIDI_BPM_MEDIAN_AVERAGE_WINDOW) / 2; i++)
+        bpm += tmpBpms[i];
+    bpm /= (float)ALSA_MIDI_BPM_MEDIAN_AVERAGE_WINDOW;
+
+    midi.prevClockUs = clock;
+
+    midi.clockCount++;
+
+    float inc = midi.clockCount / (float)MIDI_CLOCKS_PER_BEAT;
+
+    BeatTracker::BeatValues beats
+    {
+        midi.lastDivSongBeat + inc,
+        midi.lastDivMonotonicBeat + inc,
+        bpm
+    };
+
+    beats = beatTracker->setBeatValues(beats);
+
+    if (midi.clockCount >= MIDI_CLOCK_DIVISION) {
+        // Possibly preserve wrapped around beat values, if we are on the start
+        // of a clock division.
+        midi.lastDivSongBeat = beats.songBeat;
+        midi.lastDivMonotonicBeat = beats.monotonicBeat;
+        midi.clockCount = 0;
+    }
+}
+
 #endif
