@@ -34,7 +34,10 @@ using func::asString;
 using func::asHexString;
 
 
-JackEngine::JackEngine(SynthEngine *_synth) : MusicIO(_synth), jackClient(NULL)
+JackEngine::JackEngine(SynthEngine *_synth, BeatTracker *_beatTracker) :
+    MusicIO(_synth, _beatTracker),
+    jackClient(NULL),
+    internalbuff(0)
 {
     audio.jackSamplerate = 0;
     audio.jackNframes = 0;
@@ -43,7 +46,7 @@ JackEngine::JackEngine(SynthEngine *_synth) : MusicIO(_synth), jackClient(NULL)
         audio.ports[i] = NULL;
         audio.portBuffs[i] = NULL;
     }
-    midi.port = NULL;
+    midiPort = NULL;
 }
 
 
@@ -53,7 +56,7 @@ bool JackEngine::connectServer(std::string server)
     {
         if (!openJackClient(server) && tries < 2)
         {
-            synth->getRuntime().Log("Failed to open jack client, trying again", 1);
+            synth->getRuntime().Log("Failed to open jack client, trying again", _SYS_::LogError);
             usleep(3333);
         }
     }
@@ -83,7 +86,7 @@ bool JackEngine::openJackClient(std::string server)
     //Andrew Deryabin: for multi-instance support add unique id to
     //instances other then default (0)
     unsigned int synthUniqueId = synth->getUniqueId();
-    if(synthUniqueId > 0)
+    if (synthUniqueId > 0)
     {
         char sUniqueId [256];
         memset(sUniqueId, 0, sizeof(sUniqueId));
@@ -137,8 +140,8 @@ bool JackEngine::Start(void)
     internalbuff = synth->getRuntime().Buffersize;
     jack_set_xrun_callback(jackClient, _xrunCallback, this);
     #if defined(JACK_SESSION)
-        if (jack_set_session_callback
-            && jack_set_session_callback(jackClient, _jsessionCallback, this))
+        //if (jack_set_session_callback &&
+        if(jack_set_session_callback(jackClient, _jsessionCallback, this))
             synth->getRuntime().Log("Set jack session callback failed");
     #endif
 
@@ -172,16 +175,20 @@ bool JackEngine::Start(void)
      * Shows identical results but doesn't connect.
      * Original 1.4.1 version also fails - it used to work.
      */
+     /* pre V 1.3.0 was this:
+     if (Runtime.midiEngine  == jack_midi and jack_connect(jackClient,Runtime.midiDevice.c_str(),jack_port_name(midi.port)))
+         Runtime.Log("Didn't find jack MIDI source '" + Runtime.midiDevice + "'");
+    */
 
     // style-wise I think the next bit is the wrong place
     /*if (synth->getRuntime().midiEngine  == jack_midi
       && !synth->getRuntime().midiDevice.empty()
       && synth->getRuntime().midiDevice != "default")
     {
-        if(jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midi.port)))
+        if (jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midiPort)))
         {
             synth->getRuntime().Log("Didn't find jack MIDI source '"
-            + synth->getRuntime().midiDevice + "'", 1);
+            + synth->getRuntime().midiDevice + "'", _SYS_::LogError);
             synth->getRuntime().midiDevice = "";
         }
     }*/
@@ -195,7 +202,7 @@ bail_out:
 
 void JackEngine::Close(void)
 {
-    if(synth->getRuntime().runSynth)
+    if (synth->getRuntime().runSynth)
     {
         synth->getRuntime().runSynth = false;
     }
@@ -209,11 +216,11 @@ void JackEngine::Close(void)
                 jack_port_unregister(jackClient, audio.ports[chan]);
             audio.ports[chan] = NULL;
         }
-        if (NULL != midi.port)
+        if (NULL != midiPort)
         {
-            if ((chk = jack_port_unregister(jackClient, midi.port)))
+            if ((chk = jack_port_unregister(jackClient, midiPort)))
                 synth->getRuntime().Log("Failed to close jack client, status: " + asString(chk));
-            midi.port = NULL;
+            midiPort = NULL;
         }
         chk = jack_deactivate(jackClient);
         if (chk)
@@ -227,9 +234,9 @@ void JackEngine::Close(void)
 void JackEngine::registerAudioPort(int partnum)
 {
     int portnum = partnum * 2;
-    if(partnum >=0 && partnum < NUM_MIDI_PARTS)
+    if (partnum >=0 && partnum < NUM_MIDI_PARTS)
     {
-        if(audio.ports [portnum] != NULL)
+        if (audio.ports [portnum] != NULL)
         {
             synth->getRuntime().Log("Jack port " + asString(partnum) + " already registered!", 2);
             return;
@@ -239,14 +246,14 @@ void JackEngine::registerAudioPort(int partnum)
          * direct O/P.
          */
         std::string portName;
-        if(synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1))
+        if (synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1))
         {
             portName = "track_" + asString(partnum + 1) + "_l";
             audio.ports[portnum] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
             portName = "track_" + asString(partnum + 1) + "_r";
             audio.ports[portnum + 1] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 
-            if(audio.ports [portnum])
+            if (audio.ports [portnum])
             {
                 synth->getRuntime().Log("Registered jack port " + asString(partnum + 1));
             }
@@ -261,9 +268,9 @@ void JackEngine::registerAudioPort(int partnum)
 
 bool JackEngine::openAudio(void)
 {
-    if(jackClient == 0)
+    if (jackClient == 0)
     {
-        if(!connectServer(synth->getRuntime().audioDevice))
+        if (!connectServer(synth->getRuntime().audioDevice))
         {
             return false;
         }
@@ -287,26 +294,28 @@ bool JackEngine::openAudio(void)
 
 bool JackEngine::openMidi(void)
 {
-    if(jackClient == 0)
+    synth->setBPMAccurate(true);
+
+    if (jackClient == 0)
     {
-        if(!connectServer(synth->getRuntime().midiDevice))
+        if (!connectServer(synth->getRuntime().midiDevice))
         {
             return false;
         }
     }
 
     const char *port_name = "midi in";
-    midi.port = jack_port_register(jackClient, port_name,
+    midiPort = jack_port_register(jackClient, port_name,
                                    JACK_DEFAULT_MIDI_TYPE,
                                    JackPortIsInput, 0);
-    if (!midi.port)
+    if (!midiPort)
     {
         synth->getRuntime().Log("Failed to register jack midi port");
         return false;
     }
 
-    std::cout << "client " << jackClient<< "  device " << synth->getRuntime().midiDevice << "  port " << jack_port_name(midi.port) << std::endl;
-    if(jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midi.port)))
+    std::cout << "client " << jackClient<< "  device " << synth->getRuntime().midiDevice << "  port " << jack_port_name(midiPort) << std::endl;
+    if (jack_connect(jackClient, synth->getRuntime().midiDevice.c_str(), jack_port_name(midiPort)))
         {
             synth->getRuntime().Log("Didn't find jack MIDI source '"
             + synth->getRuntime().midiDevice + "'");
@@ -332,7 +341,7 @@ bool JackEngine::connectJackPorts(void)
         const char *port_name = jack_port_name(audio.ports[port + NUM_MIDI_PARTS * 2]);
         if ((ret = jack_connect(jackClient, port_name, playback_ports[port])))
         {
-            if(ret == EEXIST)
+            if (ret == EEXIST)
             {
             synth->getRuntime().Log(std::string(port_name)
                         + " is already connected to jack port " + std::string(playback_ports[port])
@@ -381,9 +390,11 @@ int JackEngine::processCallback(jack_nframes_t nframes)
     bool okaudio = true;
     bool okmidi = true;
 
-    if (midi.port)
+    if (midiPort) {
         // input exists, using jack midi
+        handleBeatValues(nframes);
         okmidi = processMidi(nframes);
+    }
     if (audio.ports[NUM_MIDI_PARTS * 2] && audio.ports[NUM_MIDI_PARTS * 2 + 1])
         // (at least) main outputs exist, using jack audio
         okaudio = processAudio(nframes);
@@ -396,7 +407,7 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
     // Part buffers
     for (int port = 0; port < 2 * NUM_MIDI_PARTS; ++port)
     {
-        if(audio.ports [port])
+        if (audio.ports [port])
         {
             audio.portBuffs[port] =
                     (float*)jack_port_get_buffer(audio.ports[port], nframes);
@@ -421,9 +432,11 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
         return false;
     }
 
+    BeatTracker::BeatValues beats(beatTracker->getBeatValues());
     int framesize;
-    if (nframes < internalbuff)
+    if (nframes <= internalbuff)
     {
+        synth->setBeatValues(beats.songBeat, beats.monotonicBeat, beats.bpm);
         framesize = sizeof(float) * nframes;
         synth->MasterAudio(zynLeft, zynRight, nframes);
         sendAudio(framesize, 0);
@@ -433,8 +446,10 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
         framesize = sizeof(float) * internalbuff;
         for (unsigned int pos = 0; pos < nframes; pos += internalbuff)
         {
-        synth->MasterAudio(zynLeft, zynRight, internalbuff);
-        sendAudio(framesize, pos);
+            float bpmInc = (float)pos * beats.bpm / (audio.jackSamplerate * 60.0f);
+            synth->setBeatValues(beats.songBeat + bpmInc, beats.monotonicBeat + bpmInc, beats.bpm);
+            synth->MasterAudio(zynLeft, zynRight, internalbuff);
+            sendAudio(framesize, pos);
         }
     }
     return true;
@@ -447,7 +462,7 @@ void JackEngine::sendAudio(int framesize, unsigned int offset)
     int currentmax = synth->getRuntime().NumAvailableParts;
     for (int port = 0, idx = 0; idx < 2 * NUM_MIDI_PARTS; port++ , idx += 2)
     {
-        if(audio.ports [idx])
+        if (audio.ports [idx])
         {
             if (jack_port_connected(audio.ports[idx])) // just a few % improvement.
             {
@@ -476,7 +491,7 @@ void JackEngine::sendAudio(int framesize, unsigned int offset)
 
 bool JackEngine::processMidi(jack_nframes_t nframes)
 {
-    void *portBuf = jack_port_get_buffer(midi.port, nframes);
+    void *portBuf = jack_port_get_buffer(midiPort, nframes);
     if (!portBuf)
     {
         synth->getRuntime().Log("Bad midi jack_port_get_buffer");
@@ -487,9 +502,9 @@ bool JackEngine::processMidi(jack_nframes_t nframes)
     jack_midi_event_t jEvent;
     jack_nframes_t eventCount = jack_midi_get_event_count(portBuf);
 
-    for(idx = 0; idx < eventCount; ++idx)
+    for (idx = 0; idx < eventCount; ++idx)
     {
-        if(!jack_midi_event_get(&jEvent, portBuf, idx))
+        if (!jack_midi_event_get(&jEvent, portBuf, idx))
         {
             if (jEvent.size >= 1 && jEvent.size <= 4) // no interest in zero sized or long events
             {
@@ -501,10 +516,41 @@ bool JackEngine::processMidi(jack_nframes_t nframes)
     return true;
 }
 
+void JackEngine::handleBeatValues(jack_nframes_t nframes)
+{
+    jack_position_t pos;
+    jack_transport_state_t state = jack_transport_query(jackClient, &pos);
+
+    BeatTracker::BeatValues beats(beatTracker->getRawBeatValues());
+
+    if (pos.valid & JackPositionBBT)
+        beats.bpm = pos.beats_per_minute;
+    else
+        beats.bpm = synth->PbpmFallback;
+
+    float bpmInc = (float)nframes * beats.bpm
+        / ((float)audio.jackSamplerate * 60.0f);
+
+    beats.monotonicBeat += bpmInc;
+
+    if (!(pos.valid & JackPositionBBT) || state == JackTransportStopped)
+        // If stopped, keep oscillating.
+        beats.songBeat += bpmInc;
+    else
+    {
+        // If rolling, sync to exact beat.
+        beats.songBeat = (float)pos.tick / (float)pos.ticks_per_beat;
+        beats.songBeat += pos.beat - 1;
+        beats.songBeat += (pos.bar - 1) * pos.beats_per_bar;
+    }
+
+    beatTracker->setBeatValues(beats);
+}
+
 
 int JackEngine::_xrunCallback(void *arg)
 {
-    ((JackEngine *)arg)->synth->getRuntime().Log("xrun reported", 2);
+    ((JackEngine *)arg)->synth->getRuntime().Log("xrun reported", _SYS_::LogNotSerious);
     return 0;
 }
 
