@@ -5,10 +5,10 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2017-2019 Will Godfrey
-    Copyright 2020 Kristian Amlie, Will Godfrey
+    Copyright 2020-2023 Kristian Amlie, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -28,15 +28,18 @@
 
 #include <cmath>
 #include <stdlib.h>
-
-using namespace std;
-
+#include "Misc/NumericFuncs.h"
+#include "Misc/SynthEngine.h"
 #include "Params/ADnoteParameters.h"
 
+using func::setAllPan;
+using func::power;
+
+
 int ADnoteParameters::ADnote_unison_sizes[] =
 {2, 3, 4, 5, 6, 8, 10, 12, 15, 20, 25, 30, 40, 50, 0};
 
-ADnoteParameters::ADnoteParameters(FFTwrapper *fft_, SynthEngine *_synth) :
+ADnoteParameters::ADnoteParameters(fft::Calc& fft_, SynthEngine *_synth) :
     Presets(_synth),
     fft(fft_)
 {
@@ -74,8 +77,10 @@ void ADnoteParameters::defaults(void)
 
     // Amplitude Global Parameters
     GlobalPar.PVolume = 90;
-    setGlobalPan(GlobalPar.PPanning = 64); // center
+    setGlobalPan(GlobalPar.PPanning = 64, synth->getRuntime().panLaw); // center
     GlobalPar.PAmpVelocityScaleFunction = 64;
+    GlobalPar.PRandom = false;
+    GlobalPar.PWidth = 63;
     GlobalPar.AmpEnvelope->defaults();
     GlobalPar.AmpLfo->defaults();
     GlobalPar.Fadein_adjustment = FADEIN_ADJUSTMENT_SCALE;
@@ -108,8 +113,8 @@ void ADnoteParameters::defaults(int n)
     VoicePar[nvoice].Unison_size = 1;
     VoicePar[nvoice].Unison_frequency_spread = 60;
     VoicePar[nvoice].Unison_stereo_spread = 64;
-    VoicePar[nvoice].Unison_vibratto = 64;
-    VoicePar[nvoice].Unison_vibratto_speed = 64;
+    VoicePar[nvoice].Unison_vibrato = 64;
+    VoicePar[nvoice].Unison_vibrato_speed = 64;
     VoicePar[nvoice].Unison_invert_phase = 0;
     VoicePar[nvoice].Unison_phase_randomness = 127;
 
@@ -127,7 +132,9 @@ void ADnoteParameters::defaults(int n)
     VoicePar[nvoice].PDelay = 0;
     VoicePar[nvoice].PVolume = 100;
     VoicePar[nvoice].PVolumeminus = 0;
-    setVoicePan(nvoice, VoicePar[nvoice].PPanning = 64); // center
+    setVoicePan(nvoice, VoicePar[nvoice].PPanning = 64, synth->getRuntime().panLaw); // center
+    VoicePar[nvoice].PRandom = false;
+    VoicePar[nvoice].PWidth = 63;
     VoicePar[nvoice].PDetune = 8192; // 8192 = 0
     VoicePar[nvoice].PCoarseDetune = 0;
     VoicePar[nvoice].PDetuneType = 0;
@@ -142,6 +149,8 @@ void ADnoteParameters::defaults(int n)
     VoicePar[nvoice].PFilterVelocityScale = 0;
     VoicePar[nvoice].PFilterVelocityScaleFunction = 64;
     VoicePar[nvoice].PFMEnabled = 0;
+    VoicePar[nvoice].PFMEnabled = 0;
+    VoicePar[nvoice].PFMringToSide = false;
     VoicePar[nvoice].PFMFixedFreq = false;
 
     // I use the internal oscillator (-1)
@@ -179,8 +188,8 @@ void ADnoteParameters::defaults(int n)
 // Init the voice parameters
 void ADnoteParameters::enableVoice(int nvoice)
 {
-    VoicePar[nvoice].POscil = new OscilParameters(synth);
-    VoicePar[nvoice].POscilFM = new OscilParameters(synth);
+    VoicePar[nvoice].POscil = new OscilParameters(fft, synth);
+    VoicePar[nvoice].POscilFM = new OscilParameters(fft, synth);
     VoicePar[nvoice].OscilSmp = new OscilGen(fft, GlobalPar.Reson, synth, VoicePar[nvoice].POscil);
     VoicePar[nvoice].FMSmp = new OscilGen(fft, NULL, synth, VoicePar[nvoice].POscilFM);
 
@@ -208,7 +217,7 @@ void ADnoteParameters::enableVoice(int nvoice)
 float ADnoteParameters::getBandwidthDetuneMultiplier(void)
 {
     float bw = (GlobalPar.PBandwidth - 64.0f) / 64.0f;
-    bw = powf(2.0f, bw * pow(fabs(bw), 0.2f) * 5.0f);
+    bw = power<2>(bw * pow(fabs(bw), 0.2f) * 5.0f);
     return bw;
 }
 
@@ -261,29 +270,21 @@ ADnoteParameters::~ADnoteParameters()
 }
 
 
-void ADnoteParameters::setGlobalPan(char pan)
+void ADnoteParameters::setGlobalPan(char pan, unsigned char panLaw)
 {
     GlobalPar.PPanning = pan;
-    if (!randomGlobalPan())
-    {
-        float t = (float)(GlobalPar.PPanning - 1) / 126.0f;
-        GlobalPar.pangainL = cosf(t * HALFPI);
-        GlobalPar.pangainR = cosf((1.0f - t) * HALFPI);
-    }
+    if (!GlobalPar.PRandom)
+        setAllPan(GlobalPar.PPanning, GlobalPar.pangainL, GlobalPar.pangainR, panLaw);
     else
         GlobalPar.pangainL = GlobalPar.pangainR = 0.7f;
 }
 
 
-void ADnoteParameters::setVoicePan(int nvoice, char pan)
+void ADnoteParameters::setVoicePan(int nvoice, char pan, unsigned char panLaw)
 {
     VoicePar[nvoice].PPanning = pan;
-    if (!randomVoicePan(nvoice))
-    {
-        float t = (float)(VoicePar[nvoice].PPanning - 1) / 126.0f;
-        VoicePar[nvoice].pangainL = cosf(t * HALFPI);
-        VoicePar[nvoice].pangainR = cosf((1.0f - t) * HALFPI);
-    }
+    if (!VoicePar[nvoice].PRandom)
+        setAllPan(VoicePar[nvoice].PPanning, VoicePar[nvoice].pangainL, VoicePar[nvoice].pangainR, panLaw);
     else
         VoicePar[nvoice].pangainL = VoicePar[nvoice].pangainR = 0.7f;
 }
@@ -295,6 +296,8 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
     if (nvoice >= NUM_VOICES)
         return;
 
+    // currently not used
+    // bool yoshiFormat = synth->usingYoshiType;
     int oscilused = 0, fmoscilused = 0; // if the oscil or fmoscil are used by another voice
 
     for (int i = 0; i < NUM_VOICES; ++i)
@@ -310,26 +313,25 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
 
     xml->addpar("type", VoicePar[nvoice].Type);
 
-    xml->addpar("unison_size", VoicePar[nvoice].Unison_size);
-    xml->addpar("unison_frequency_spread",
-                VoicePar[nvoice].Unison_frequency_spread);
-    xml->addpar("unison_stereo_spread", VoicePar[nvoice].Unison_stereo_spread);
-    xml->addpar("unison_vibratto", VoicePar[nvoice].Unison_vibratto);
-    xml->addpar("unison_vibratto_speed", VoicePar[nvoice].Unison_vibratto_speed);
-    xml->addpar("unison_invert_phase", VoicePar[nvoice].Unison_invert_phase);
+    xml->addpar("unison_size",             VoicePar[nvoice].Unison_size);
+    xml->addpar("unison_frequency_spread", VoicePar[nvoice].Unison_frequency_spread);
+    xml->addpar("unison_stereo_spread",    VoicePar[nvoice].Unison_stereo_spread);
+    xml->addpar("unison_vibratto",         VoicePar[nvoice].Unison_vibrato);
+    xml->addpar("unison_vibratto_speed",   VoicePar[nvoice].Unison_vibrato_speed);
+    xml->addpar("unison_invert_phase",     VoicePar[nvoice].Unison_invert_phase);
     xml->addpar("unison_phase_randomness", VoicePar[nvoice].Unison_phase_randomness);
 
     xml->addpar("delay", VoicePar[nvoice].PDelay);
     xml->addparbool("resonance", VoicePar[nvoice].Presonance);
 
     xml->addpar("input_voice", VoicePar[nvoice].PVoice);
-    xml->addpar("ext_oscil", VoicePar[nvoice].Pextoscil);
-    xml->addpar("ext_fm_oscil", VoicePar[nvoice].PextFMoscil);
+    xml->addpar("ext_oscil",   VoicePar[nvoice].Pextoscil);
+    xml->addpar("ext_fm_oscil",VoicePar[nvoice].PextFMoscil);
 
     xml->addpar("oscil_phase", VoicePar[nvoice].Poscilphase);
     xml->addpar("oscil_fm_phase", VoicePar[nvoice].PFMoscilphase);
 
-    xml->addparbool("filter_enabled", VoicePar[nvoice].PFilterEnabled);
+    xml->addparbool("filter_enabled",VoicePar[nvoice].PFilterEnabled);
     xml->addparbool("filter_bypass", VoicePar[nvoice].Pfilterbypass);
 
     xml->addpar("fm_enabled", VoicePar[nvoice].PFMEnabled);
@@ -340,7 +342,17 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
 
 
     xml->beginbranch("AMPLITUDE_PARAMETERS");
-        xml->addpar("panning", VoicePar[nvoice].PPanning);
+        // new yoshi type
+        xml->addpar("pan_pos", VoicePar[nvoice].PPanning);
+        xml->addparbool("random_pan", VoicePar[nvoice].PRandom);
+        xml->addpar("random_width", VoicePar[nvoice].PWidth);
+
+        // legacy
+        if (VoicePar[nvoice].PRandom)
+            xml->addpar("panning", 0);
+        else
+            xml->addpar("panning", VoicePar[nvoice].PPanning);
+
         xml->addpar("volume", VoicePar[nvoice].PVolume);
         xml->addparbool("volume_minus", VoicePar[nvoice].PVolumeminus);
         xml->addpar("velocity_sensing", VoicePar[nvoice].PAmpVelocityScaleFunction);
@@ -455,13 +467,25 @@ void ADnoteParameters::add2XMLsection(XMLwrapper *xml, int n)
 
 void ADnoteParameters::add2XML(XMLwrapper *xml)
 {
+    // currently not used
+    // bool yoshiFormat = synth->usingYoshiType;
     xml->information.ADDsynth_used = 1;
 
     xml->addparbool("stereo", GlobalPar.PStereo);
 
     xml->beginbranch("AMPLITUDE_PARAMETERS");
         xml->addpar("volume", GlobalPar.PVolume);
-        xml->addpar("panning", GlobalPar.PPanning);
+        // new yoshi type
+        xml->addpar("pan_pos", GlobalPar.PPanning);
+        xml->addparbool("random_pan", GlobalPar.PRandom);
+        xml->addpar("random_width", GlobalPar.PWidth);
+
+        // legacy
+        if (GlobalPar.PRandom)
+            xml->addpar("panning", 0);
+        else
+            xml->addpar("panning", GlobalPar.PPanning);
+
         xml->addpar("velocity_sensing", GlobalPar.PAmpVelocityScaleFunction);
         xml->addpar("fadein_adjustment", GlobalPar.Fadein_adjustment);
         xml->addpar("punch_strength", GlobalPar.PPunchStrength);
@@ -534,7 +558,27 @@ void ADnoteParameters::getfromXML(XMLwrapper *xml)
     if (xml->enterbranch("AMPLITUDE_PARAMETERS"))
     {
         GlobalPar.PVolume = xml->getpar127("volume", GlobalPar.PVolume);
-        setGlobalPan(xml->getpar127("panning", GlobalPar.PPanning));
+        int test = xml->getpar127("random_width", UNUSED);
+        if (test < 64) // new Yoshi type
+        {
+            GlobalPar.PWidth = test;
+            setGlobalPan(xml->getpar127("pan_pos", GlobalPar.PPanning), synth->getRuntime().panLaw);
+            GlobalPar.PRandom = xml->getparbool("random_pan", GlobalPar.PRandom);
+        }
+        else // legacy
+        {
+            setGlobalPan(xml->getpar127("panning", GlobalPar.PPanning), synth->getRuntime().panLaw);
+
+            if (GlobalPar.PPanning == 0)
+            {
+                GlobalPar.PPanning = 64;
+                GlobalPar.PRandom = true;
+                GlobalPar.PWidth = 63;
+            }
+            else
+                GlobalPar.PRandom = false;
+        }
+
         GlobalPar.PAmpVelocityScaleFunction =
             xml->getpar127("velocity_sensing", GlobalPar.PAmpVelocityScaleFunction);
         GlobalPar.Fadein_adjustment = xml->getpar127("fadein_adjustment", GlobalPar.Fadein_adjustment);
@@ -624,8 +668,8 @@ void ADnoteParameters::getfromXML(XMLwrapper *xml)
 
 void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 {
-    int nvoice=n;
-    if (nvoice>=NUM_VOICES) return;
+    int nvoice = n;
+    if (nvoice >= NUM_VOICES) return;
 
     VoicePar[nvoice].Enabled = xml->getparbool("enabled", 0);
 
@@ -635,10 +679,10 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
         xml->getpar127("unison_frequency_spread", VoicePar[nvoice].Unison_frequency_spread);
     VoicePar[nvoice].Unison_stereo_spread =
         xml->getpar127("unison_stereo_spread", VoicePar[nvoice].Unison_stereo_spread);
-    VoicePar[nvoice].Unison_vibratto =
-        xml->getpar127("unison_vibratto", VoicePar[nvoice].Unison_vibratto);
-    VoicePar[nvoice].Unison_vibratto_speed =
-        xml->getpar127("unison_vibratto_speed", VoicePar[nvoice].Unison_vibratto_speed);
+    VoicePar[nvoice].Unison_vibrato =
+        xml->getpar127("unison_vibratto", VoicePar[nvoice].Unison_vibrato);
+    VoicePar[nvoice].Unison_vibrato_speed =
+        xml->getpar127("unison_vibratto_speed", VoicePar[nvoice].Unison_vibrato_speed);
     VoicePar[nvoice].Unison_invert_phase =
         xml->getpar127("unison_invert_phase", VoicePar[nvoice].Unison_invert_phase);
     VoicePar[nvoice].Unison_phase_randomness =
@@ -673,7 +717,24 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 
     if (xml->enterbranch("AMPLITUDE_PARAMETERS"))
     {
-        setVoicePan(nvoice, xml->getpar127("panning", VoicePar[nvoice].PPanning));
+        int test = xml->getpar127("random_width", UNUSED);
+        if (test < 64) // new Yoshi type
+        {
+            VoicePar[nvoice].PWidth = test;
+            setVoicePan(nvoice, xml->getpar127("pan_pos", VoicePar[nvoice].PPanning), synth->getRuntime().panLaw);
+            VoicePar[nvoice].PRandom = xml->getparbool("random_pan", VoicePar[nvoice].PRandom);
+        }
+        else  // legacy
+        {
+            setVoicePan(nvoice, xml->getpar127("panning", VoicePar[nvoice].PPanning), synth->getRuntime().panLaw);
+            if (VoicePar[nvoice].PPanning == 0)
+            {
+                VoicePar[nvoice].PPanning = 64;
+                VoicePar[nvoice].PRandom = true;
+                VoicePar[nvoice].PWidth = 63;
+            }
+        }
+
         VoicePar[nvoice].PVolume = xml->getpar127("volume", VoicePar[nvoice].PVolume);
         VoicePar[nvoice].PVolumeminus =
             xml->getparbool("volume_minus", VoicePar[nvoice].PVolumeminus);
@@ -856,7 +917,7 @@ void ADnoteParameters::getfromXMLsection(XMLwrapper *xml, int n)
 
 float ADnoteParameters::getLimits(CommandBlock *getData)
 {
-    float value = getData->data.value.F;
+    float value = getData->data.value;
     int request = int(getData->data.type & TOPLEVEL::type::Default);
     int control = getData->data.control;
     int engine = getData->data.engine;
@@ -889,12 +950,16 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
                 def = 64;
                 break;
 
-            case ADDSYNTH::control::enable:
-                type |= learnable;
-                def = 1;
+            case ADDSYNTH::control::enableRandomPan:
                 max = 1;
                 break;
 
+            case ADDSYNTH::control::randomWidth:
+                type |= learnable;
+                def = 63;
+                max = 63;
+                break;
+
             case ADDSYNTH::control::detuneFrequency:
                 type |= learnable;
                 min = -8192;
@@ -933,20 +998,26 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
                 break;
 
             case ADDSYNTH::control::dePop:
+                type |= learnable;
                 def = FADEIN_ADJUSTMENT_SCALE;
+                break;
 
-            case ADDSYNTH::control::punchStrength: // just ensures it doesn't get caught by default
+            case ADDSYNTH::control::punchStrength:
+                type |= learnable;
                 break;
 
             case ADDSYNTH::control::punchDuration:
+                type |= learnable;
                 def = 60;
                 break;
 
             case ADDSYNTH::control::punchStretch:
+                type |= learnable;
                 def = 64;
                 break;
 
             case ADDSYNTH::control::punchVelocity:
+                type |= learnable;
                 def = 72;
                 break;
 
@@ -961,9 +1032,9 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
         switch (request)
         {
             case TOPLEVEL::type::Adjust:
-                if(value < min)
+                if (value < min)
                     value = min;
-                else if(value > max)
+                else if (value > max)
                     value = max;
             break;
             case TOPLEVEL::type::Minimum:
@@ -995,6 +1066,14 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
             type |= learnable;
             def = 64;
             break;
+        case ADDVOICE::control::enableRandomPan:
+            max = 1;
+            break;
+
+        case ADDVOICE::control::randomWidth:
+            def = 63;
+            max = 63;
+            break;
 
         case ADDVOICE::control::invertPhase:
             max = 1;
@@ -1110,6 +1189,7 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
             break;
 
         case ADDVOICE::control::enableUnison:
+            type |= learnable;
             max = 1;
             break;
 
@@ -1228,9 +1308,9 @@ float ADnoteParameters::getLimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
         break;
         case TOPLEVEL::type::Minimum:
