@@ -8,7 +8,7 @@
     Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -29,9 +29,10 @@
 #include <cmath>
 
 #include "Params/LFOParams.h"
+#include "Misc/NumericFuncs.h"
+
+using func::power;
 
-//#include <iostream>
-//int LFOParams::time = 0;
 
 LFOParams::LFOParams(float Pfreq_, unsigned char Pintensity_,
                      unsigned char Pstartphase_, unsigned char PLFOtype_,
@@ -73,6 +74,7 @@ void LFOParams::defaults(void)
     Prandomness = Drandomness;
     Pdelay = Ddelay;
     Pcontinous = Dcontinous;
+    Pbpm = 0;
     Pfreqrand = 0;
     Pstretch = 64;
 }
@@ -82,15 +84,20 @@ void LFOParams::setPfreq(int32_t n)
 {
 
     PfreqI = n;
-    Pfreq = (powf(2.0f, (float(n) / float(Fmul2I)) * 10.0f) - 1.0f) / 12.0f;
+    Pfreq = (power<2>((float(n) / float(Fmul2I)) * 10.0f) - 1.0f) / 12.0f;
     presetsUpdated();
 }
 
 
 void LFOParams::add2XML(XMLwrapper *xml)
 {
-    xml->addpar("freqI", PfreqI);
-    xml->addparreal("freq", float(PfreqI) / Fmul2I);
+    float freqF = float(PfreqI) / float(Fmul2I);
+    if (Pbpm)
+        // Save quantized, so that we can make the scale finer in the future, if
+        // necessary.
+        freqF = func::quantizedLFOfreqBPM(freqF);
+    xml->addpar("freqI", freqF * float(Fmul2I));
+    xml->addparreal("freq", freqF);
     xml->addpar("intensity", Pintensity);
     xml->addpar("start_phase", Pstartphase);
     xml->addpar("lfo_type", PLFOtype);
@@ -99,6 +106,7 @@ void LFOParams::add2XML(XMLwrapper *xml)
     xml->addpar("delay", Pdelay);
     xml->addpar("stretch", Pstretch);
     xml->addparbool("continous",    Pcontinous);
+    xml->addparbool("bpm", Pbpm);
 }
 
 
@@ -106,7 +114,7 @@ void LFOParams::getfromXML(XMLwrapper *xml)
 {
     //PfreqI = xml->getpar("freqI", -1, 0, Fmul2I);
     //if (PfreqI == -1)
-    PfreqI = xml->getparreal("freq", Pfreq, 0.0, 1.0) * Fmul2I;
+    PfreqI = xml->getparreal("freq", Pfreq, 0.0, 1.0) * float(Fmul2I);
     setPfreq(PfreqI);
 
     Pintensity = xml->getpar127("intensity", Pintensity);
@@ -117,12 +125,13 @@ void LFOParams::getfromXML(XMLwrapper *xml)
     Pdelay = xml->getpar127("delay", Pdelay);
     Pstretch = xml->getpar127("stretch", Pstretch);
     Pcontinous = xml->getparbool("continous", Pcontinous);
+    Pbpm = xml->getparbool("bpm", Pbpm);
     presetsUpdated();
 }
 
 float LFOlimit::getLFOlimits(CommandBlock *getData)
 {
-    float value = getData->data.value.F;
+    float value = getData->data.value;
     int request = int(getData->data.type & TOPLEVEL::type::Default);
     int control = getData->data.control;
     int engine = getData->data.engine;
@@ -193,13 +202,17 @@ float LFOlimit::getLFOlimits(CommandBlock *getData)
         case LFOINSERT::control::amplitudeRandomness:
             break;
         case LFOINSERT::control::type:
-            max = 6;
+            max = 9;
             type &= ~learnable;
             break;
         case LFOINSERT::control::continuous:
             max = 1;
             type &= ~learnable;
             break;
+        case LFOINSERT::control::bpm:
+            max = 1;
+            type &= ~learnable;
+            break;
         case LFOINSERT::control::frequencyRandomness:
             break;
         case LFOINSERT::control::stretch:
@@ -217,9 +230,9 @@ float LFOlimit::getLFOlimits(CommandBlock *getData)
     switch (request)
     {
         case TOPLEVEL::type::Adjust:
-            if(value < min)
+            if (value < min)
                 value = min;
-            else if(value > max)
+            else if (value > max)
                 value = max;
         break;
         case TOPLEVEL::type::Minimum:
