@@ -5,10 +5,10 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009, Alan Calvert
     Copyright 2017-2018, Will Godfrey
-    Copyright 2020 Kristian Amlie
+    Copyright 2020-2022 Kristian Amlie & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -40,8 +40,8 @@ class SUBnoteParameters : public Presets
     public:
         SUBnoteParameters(SynthEngine *_synth);
         ~SUBnoteParameters();
-        void setPan(char pan);
-        bool randomPan(void) { return !PPanning; }
+        void setPan(char pan, unsigned char panLaw);
+        //bool randomPan(void) { return !PPanning; }
         void add2XML(XMLwrapper *xml);
         void defaults(void);
         void getfromXML(XMLwrapper *xml);
@@ -52,6 +52,8 @@ class SUBnoteParameters : public Presets
         bool Pstereo; // true = stereo, false = mono
         unsigned char PVolume;
         unsigned char PPanning;
+        bool PRandom;
+        unsigned char PWidth;
         float pangainL;         // derived from PPanning
         float pangainR;         // ^^
         unsigned char PAmpVelocityScaleFunction;
@@ -101,8 +103,6 @@ class SUBnoteParameters : public Presets
         unsigned char Phmagtype;    // how the magnitudes are computed
                                     // 0 = linear, 1 = -60dB, 2 = -60dB
 
-        unsigned char PfilterChanged[MAX_SUB_HARMONICS]; // 0 = no, 6 = magnitude, 7 = bandwidth
-
         unsigned char Phmag[MAX_SUB_HARMONICS];   // Magnitudes
 
         unsigned char Phrelbw[MAX_SUB_HARMONICS]; // Relative BandWidth ("64"=1.0)
