@@ -7,7 +7,7 @@
     Copyright 2020 Kristian Amlie
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -28,13 +28,14 @@
 #include "Misc/NumericFuncs.h"
 #include "Params/EnvelopeParams.h"
 
-using func::dB2rap;
-using func::rap2dB;
+using func::power;
+using func::decibel;
+using func::asDecibel;
 
 
 Envelope::Envelope(EnvelopeParams *envpars, float basefreq_, SynthEngine *_synth):
     _envpars(envpars),
-    envUpdate(envpars),
+    envUpdate(*envpars),
     basefreq(basefreq_),
     synth(_synth)
 {
@@ -73,23 +74,15 @@ void Envelope::recomputePoints()
     int mode = _envpars->Envmode;
 
     // for amplitude envelopes
-    if (mode == ENVMODE::amplitudeLin && linearenvelope == 0)
-        mode = ENVMODE::amplitudeLog; // change to log envelope
-    if (mode == ENVMODE::amplitudeLog && linearenvelope != 0)
-        mode = ENVMODE::amplitudeLin; // change to linear
+    if (mode == PART::envelope::groupmode::amplitudeLin && linearenvelope == 0)
+        mode = PART::envelope::groupmode::amplitudeLog; // change to log envelope
+    if (mode == PART::envelope::groupmode::amplitudeLog && linearenvelope != 0)
+        mode = PART::envelope::groupmode::amplitudeLin; // change to linear
 
     envstretch = powf(440.0f / basefreq, _envpars->Penvstretch / 64.0f);
 
-    float bufferdt = synth->sent_all_buffersize_f / synth->samplerate_f;
-
     for (int i = 0; i < MAX_ENVELOPE_POINTS; ++i)
     {
-        float tmp = _envpars->getdt(i) / 1000.0f * envstretch;
-        if (tmp > bufferdt)
-            envdt[i] = bufferdt / tmp;
-        else
-            envdt[i] = 2.0f; // any value larger than 1
-
         switch (mode)
         {
             case 2:
@@ -98,7 +91,7 @@ void Envelope::recomputePoints()
 
             case 3:
                 envval[i] =
-                    (powf(2.0f, 6.0f * fabsf(_envpars->Penvval[i] - 64.0f) / 64.0f) - 1.0f) * 100.0f;
+                    (power<2>(6.0f * fabsf(_envpars->Penvval[i] - 64.0f) / 64.0f) - 1.0f) * 100.0f;
                 if (_envpars->Penvval[i] < 64)
                     envval[i] = -envval[i];
                 break;
@@ -113,10 +106,9 @@ void Envelope::recomputePoints()
 
             default:
                 envval[i] = _envpars->Penvval[i] / 127.0f;
+                break;
         }
     }
-
-    envdt[0] = 1.0f;
 }
 
 // Envelope Output
@@ -137,16 +129,22 @@ float Envelope::envout(void)
         return envoutval;
     }
 
+    float bufferdt = synth->sent_buffersize_f / synth->samplerate_f;
+
     if (keyreleased && forcedrelase)
     {   // do the forced release
         int tmp = (envsustain < 0) ? (envpoints - 1) : (envsustain + 1);
         // if there is no sustain point, use the last point for release
 
-        if (envdt[tmp] <0.00000001f)
+        float envdt = bufferdt * 1000.0f / (_envpars->getdt(tmp) * envstretch);
+        if (envdt >= 1.0f)
+            envdt = 2.0f; // any value larger than 1
+
+        if (envdt <0.00000001f)
             out = envval[tmp];
         else
             out = envoutval + (envval[tmp] - envoutval) * t;
-        t += envdt[tmp] * envstretch;
+        t += envdt * envstretch;
 
         if (t >= 1.0f)
         {
@@ -158,13 +156,18 @@ float Envelope::envout(void)
         }
         return out;
     }
-    if (envdt[currentpoint] >= 1.0f)
+
+    float envdt = bufferdt * 1000.0f / (_envpars->getdt(currentpoint) * envstretch);
+    if (envdt >= 1.0f)
+        envdt = 2.0f; // any value larger than 1
+
+    if (envdt >= 1.0f)
         out = envval[currentpoint];
     else
         out = envval[currentpoint - 1] + (envval[currentpoint]
               - envval[currentpoint - 1]) * t;
 
-    t += envdt[currentpoint];
+    t += envdt;
     if (t >= 1.0f)
     {
         if (currentpoint >= envpoints - 1)
@@ -190,12 +193,17 @@ float Envelope::envout_dB(void)
         return envout();
 
     if (currentpoint == 1 && (keyreleased == 0 || forcedrelase == 0))
-    {   // first point is always lineary interpolated
-        float v1 = dB2rap(envval[0]);
-        float v2 = dB2rap(envval[1]);
+    {   // first point is always linearly interpolated
+        float v1 = decibel(envval[0]);
+        float v2 = decibel(envval[1]);
         out = v1 + (v2 - v1) * t;
 
-        t += envdt[1];
+        float bufferdt = synth->sent_buffersize_f / synth->samplerate_f;
+        float envdt = bufferdt * 1000.0f / (_envpars->getdt(1) * envstretch);
+        if (envdt >= 1.0f)
+            envdt = 2.0f; // any value larger than 1
+
+        t += envdt;
         if (t >= 1.0f)
         {
             t = 0.0f;
@@ -204,11 +212,11 @@ float Envelope::envout_dB(void)
         }
 
         if (out > 0.001f)
-            envoutval = rap2dB(out);
+            envoutval = asDecibel(out);
         else
             envoutval = MIN_ENVELOPE_DB;
     } else
-        out = dB2rap(envout());
+        out = decibel(envout());
 
     return out;
 }
