@@ -5,10 +5,10 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2017, Will Godfrey & others
-    Copyright 2020 Kristian Amlie
+    Copyright 2020 Kristian Amlie & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -29,6 +29,7 @@
 #define LFO_H
 
 #include "Params/LFOParams.h"
+#include "Misc/NumericFuncs.h"
 
 class SynthEngine;
 
@@ -40,6 +41,10 @@ class LFO
         float lfoout(void);
         float amplfoout(void);
     private:
+        std::pair<float, float> getBpmFrac() {
+            return func::LFOfreqBPMFraction((float)lfopars->PfreqI / float(Fmul2I));
+        }
+
         LFOParams *lfopars;
         Presets::PresetsUpdate lfoUpdate;
         void Recompute(void);
@@ -53,9 +58,16 @@ class LFO
         float lfornd;
         float lfofreqrnd;
         float lfoelapsed;
+        float startPhase;
         char lfotype;
         int freqrndenabled;
 
+        float sampandholdvalue;
+        int issampled;
+
+        float prevMonotonicBeat;
+        std::pair<float, float> prevBpmFrac;
+
         SynthEngine *synth;
 };
 
