@@ -4,10 +4,10 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2017-2019 Will Godfrey & others.
+    Copyright 2017-2020 Will Godfrey & others.
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -22,7 +22,6 @@
 
     This file is derivative of original ZynAddSubFX code.
 
-    Modified May 2019
 */
 
 #ifndef OSCIL_GEN_H
@@ -30,10 +29,12 @@
 
 #include <sys/types.h>
 #include <limits.h>
+#include <vector>
 
 #include "Misc/RandomGen.h"
 #include "Misc/WaveShapeSamples.h"
 #include "Misc/XMLwrapper.h"
+#include "DSP/FFTwrapper.h"
 #include "Params/OscilParameters.h"
 #include "Synth/Resonance.h"
 
@@ -42,56 +43,68 @@ class SynthEngine;
 class OscilGen : private WaveShapeSamples
 {
     public:
-        OscilGen(FFTwrapper *fft_,Resonance *res_, SynthEngine *_synth, OscilParameters *params_);
-        ~OscilGen();
+        OscilGen(fft::Calc&,Resonance *res_, SynthEngine *_synth, OscilParameters *params_);
+       ~OscilGen() = default;
+
+        // shall not be copied or moved or assigned
+        OscilGen(OscilGen&&)                 = delete;
+        OscilGen(OscilGen const&)            = delete;
+        OscilGen& operator=(OscilGen&&)      = delete;
+        OscilGen& operator=(OscilGen const&) = delete;
+
 
         void changeParams(OscilParameters *params_);
 
         void prepare();
 
-        void get(float *smps, float freqHz);
-        // returns where should I start getting samples, used in block type randomness
-
-        void get(float *smps, float freqHz, int resonance);
-        // if freqHz is smaller than 0, return the "un-randomized" sample for UI
+        void getWave(fft::Waveform&, float freqHz, bool applyResonance =false, bool forGUI =false);
+        std::vector<float> getSpectrumForPAD(float freqHz);
 
         // Get just the phase of the oscillator.
         int getPhase();
 
-        void getbasefunction(float *smps);
+        void getbasefunction(fft::Waveform&);
 
         // called by UI
-        void getspectrum(int n, float *spc, int what); // what=0 pt. oscil,1 pt. basefunc
-        void getcurrentbasefunction(float *smps);
+        void getOscilSpectrumIntensities(size_t, float*);
+        void getBasefuncSpectrumIntensities(size_t, float*);
+        void displayBasefuncForGui(fft::Waveform&);
+        void displayWaveformForGui(fft::Waveform&);
+
+        // convert the current Oscil settings into a "user base function"
         void useasbase(void);
 
         void genDefaults(void);
         void defaults(void);
         void convert2sine();
 
-        // Make a new random seed for Amplitude Randomness -
-        //   should be called every noteon event
-        void newrandseed() { randseed = prng.randomINT() + INT_MAX/2; }
+        // Draw a new random seed for randomisation of harmonics - called every noteon event
+        void newrandseed() { randseed = basePrng.randomINT() + INT_MAX/2; }
+        void resetHarmonicPrng() { harmonicPrng.init(randseed); }
+        void reseed(int value);
+        void forceUpdate();
 
     private:
         OscilParameters *params;
 
         SynthEngine *synth;
 
-        float *tmpsmps;
-        FFTFREQS outoscilFFTfreqs;
+        fft::Calc& fft;
+
+        fft::Waveform tmpsmps;
 
         float hmag[MAX_AD_HARMONICS], hphase[MAX_AD_HARMONICS];
         // the magnituides and the phases of the sine/nonsine harmonics
 
-        FFTwrapper *fft;
+        // OscilGen core implementation: generate the current Spectrum -> outoscilSpectrum
+        void buildSpectrum(float freqHz, bool applyResonance, bool forGUI, bool forPAD);
 
-        // computes the basefunction and make the FFT; newbasefunc<0  = same basefunc
+        // computes the basefunction and make the FFT;
         void changebasefunction(void);
 
         void waveshape(void); // Waveshaping (no kidding!)
 
-        void oscilfilter(); // Filter the oscillator accotding to Pfiltertype and Pfilterpar
+        void oscilfilter(); // Filter the oscillator according to Pfiltertype and Pfilterpar
 
         void spectrumadjust(void); // Adjust the spectrum
 
@@ -99,15 +112,6 @@ class OscilGen : private WaveShapeSamples
 
         void modulation(void); // Do the oscil modulation stuff
 
-        void adaptiveharmonic(FFTFREQS f, float freq);
-        // Do the adaptive harmonic stuff
-
-        // Do the adaptive harmonic postprocessing (2n+1,2xS,2xA,etc..)
-        // this function is called even for the user interface
-        // this can be called for the sine and components, and for the spectrum
-        // (that's why the sine and cosine components should be processed with
-        // a separate call)
-        void adaptiveharmonicpostprocess(float *f, int size);
 
         // Basic/base functions (Functiile De Baza)
         float basefunc_pulse(float x, float a);
@@ -125,6 +129,8 @@ class OscilGen : private WaveShapeSamples
         float basefunc_sqr(float x, float a);
         float basefunc_spike(float x, float a);
         float basefunc_circle(float x, float a);
+        float basefunc_hypsec(float x, float a);
+
 
         // Internal Data
         unsigned char oldbasefunc,
@@ -146,10 +152,12 @@ class OscilGen : private WaveShapeSamples
             oldmodulationpar2,
             oldmodulationpar3;
 
-        FFTFREQS oscilFFTfreqs; // Oscillator Frequencies - this is different
-                                // than the hamonics set-up by the user, it may
-                                // contain time-domain data if the antialiasing
-                                // is turned off
+        fft::Spectrum outoscilSpectrum;
+
+        fft::Spectrum oscilSpectrum; // Oscillator Frequencies - this is different
+                                     // than the hamonics set-up by the user, it may
+                                     // contain time-domain data if the antialiasing
+                                     // is turned off
         Presets::PresetsUpdate oscilupdate;// whether the oscil is prepared, if
                                            // not prepared we need to call
                                            // ::prepare() before ::get()
@@ -159,8 +167,15 @@ class OscilGen : private WaveShapeSamples
 
         uint32_t randseed;
 
-        RandomGen prng;
+        RandomGen basePrng;
         RandomGen harmonicPrng;
 };
 
+// allow to mark this OscilGen as "dirty" to force recalculation of spectrum
+// (as of 4/22 only relevant for automated testing, see SynthEngine::setReproducibleState()
+inline void OscilGen::forceUpdate()
+{
+    oscilupdate.forceUpdate();
+}
+
 #endif
