@@ -5,10 +5,10 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2010 Alan Calvert
     Copyright 2014-2017 Will Godfrey & others
-    Copyright 2020 Kristian Amlie
+    Copyright 2020 Kristian Amlie & others
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -28,7 +28,13 @@
 #ifndef SUB_NOTE_H
 #define SUB_NOTE_H
 
-#include "Synth/LegatoTypes.h"
+#include "globals.h"
+#include "Misc/Alloc.h"
+#include "Params/Presets.h"
+
+#include <memory>
+
+using std::unique_ptr;
 
 class SUBnoteParameters;
 class Controller;
@@ -40,59 +46,69 @@ class SynthEngine;
 class SUBnote
 {
     public:
-        SUBnote(SUBnoteParameters *parameters, Controller *ctl_,
-                float freq, float velocity, int portamento_,
-                int midinote, bool besilent, SynthEngine *_synth);
-        ~SUBnote();
+        SUBnote(SUBnoteParameters& parameters, Controller& ctl_, Note, bool portamento_);
+        SUBnote(SUBnote const&);
+       ~SUBnote();
 
-        void SUBlegatonote(float freq, float velocity,
-                           int portamento_, int midinote, bool externcall);
+        // shall not be moved or assigned
+        SUBnote(SUBnote&&)                 = delete;
+        SUBnote& operator=(SUBnote&&)      = delete;
+        SUBnote& operator=(SUBnote const&) = delete;
 
-        int noteout(float *outl,float *outr); // note output, return 0 if the
-                                              // note is finished
+        void performPortamento(Note);
+        void legatoFadeIn(Note);
+        void legatoFadeOut();
+
+        void noteout(float *outl,float *outr);
         void releasekey(void);
-        bool finished(void) { return !NoteEnabled; }
-
-        bool ready; // if I can get the sampledata
+        bool finished() const { return noteStatus == NOTE_DISABLED; }
 
     private:
         void computecurrentparameters(void);
         void initparameters(float freq);
-        void KillNote(void);
+        void killNote(void);
         void updatefilterbank(void);
 
-        SUBnoteParameters *pars;
 
+        SynthEngine& synth;
+        SUBnoteParameters& pars;
+        Presets::PresetsUpdate subNoteChange;
+        Controller& ctl;
+
+        Note note;
         bool stereo;
+        float realfreq;
+        bool portamento;
+        int numstages;              // number of stages of filters
+        int numharmonics;           // number of harmonics (after the too higher harmonics are removed)
+        int start;                  // how the harmonics start
         int pos[MAX_SUB_HARMONICS]; // chart of non-zero harmonic locations
-        int numstages; // number of stages of filters
-        int numharmonics; // number of harmonics (after the too higher hamonics are removed)
-        int start; // how the harmonics start
-        float basefreq;
-        float BendAdjust;
-        float OffsetHz;
+        float bendAdjust;
+        float offsetHz;
         float randpanL;
         float randpanR;
 
-        Envelope *AmpEnvelope;
-        Envelope *FreqEnvelope;
-        Envelope *BandWidthEnvelope;
+        unique_ptr<Envelope> ampEnvelope;
+        unique_ptr<Envelope> freqEnvelope;
+        unique_ptr<Envelope> bandWidthEnvelope;
+        unique_ptr<Envelope> globalFilterEnvelope;
 
-        Filter *GlobalFilterL,*GlobalFilterR;
+        unique_ptr<Filter> globalFilterL;
+        unique_ptr<Filter> globalFilterR;
 
-        Envelope *GlobalFilterEnvelope;
 
         // internal values
-        bool NoteEnabled;
+        enum NoteStatus {
+            NOTE_DISABLED,
+            NOTE_ENABLED,
+            NOTE_LEGATOFADEOUT
+        } noteStatus;
+
         int firsttick;
-        int portamento;
         float volume;
         float oldamplitude;
         float newamplitude;
 
-        float GlobalFilterCenterPitch; // octaves
-        float GlobalFilterFreqTracking;
-
         struct bpfilter {
             float freq;
             float bw;
@@ -116,56 +132,28 @@ class SUBnote
         void computeallfiltercoefs();
         void computefiltercoefs(bpfilter &filter, float freq, float bw, float gain);
         void computeNoteParameters();
-        void setBaseFreq();
+        float computeRealFreq();
         void filter(bpfilter &filter, float *smps);
         void filterVarRun(bpfilter &filter, float *smps);
         float getHgain(int harmonic);
 
-        bpfilter *lfilter;
-        bpfilter *rfilter;
+        unique_ptr<bpfilter[]> lfilter;
+        unique_ptr<bpfilter[]> rfilter;
 
         float overtone_rolloff[MAX_SUB_HARMONICS];
         float overtone_freq[MAX_SUB_HARMONICS];
 
-        float *tmpsmp;
-        float *tmprnd; // this is filled with random numbers
+        Samples& tmpsmp;
+        Samples& tmprnd; // this is filled with random numbers
 
-        Controller *ctl;
         int oldpitchwheel;
         int oldbandwidth;
-        float globalfiltercenterq;
 
         // Legato vars
-        struct {
-            bool silent;
-            float lastfreq;
-            LegatoMsg msg;
-            int decounter;
-            struct {
-                // Fade In/Out vars
-                int length;
-                float m;
-                float step;
-            } fade;
+        float legatoFade;
+        float legatoFadeStep;
 
-            struct {
-                // Note parameters
-                float freq;
-                float vel;
-                int portamento;
-                int midinote;
-            } param;
-        } Legato;
-
-        const float log_0_01;    // logf(0.01);
-        const float log_0_001;   // logf(0.001);
-        const float log_0_0001;  // logf(0.0001);
-        const float log_0_00001; // logf(0.00001);
-
-        Presets::PresetsUpdate subNoteChange;
-
-        SynthEngine *synth;
         int filterStep;
 };
+#endif /*SUB_NOTE_H*/
 
-#endif
