@@ -6,7 +6,7 @@
     Copyright 2017 Jesper Lloyd
 
     This file is part of yoshimi, which is free software: you can redistribute
-    it and/or modify it under the terms of the GNU Library General Public
+    it and/or modify it under the terms of the GNU General Public
     License as published by the Free Software Foundation; either version 2 of
     the License, or (at your option) any later version.
 
@@ -43,7 +43,7 @@ class mwheel_val_slider : public Fl_Value_Slider {
   /* Overridden widget methods */
   int handle(int ev);
   void tooltip(const char* tip);
-  void value(double);
+  int value(double);
   double value();
 
  protected:
