@@ -2,7 +2,7 @@
     main.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2020, Will Godfrey & others
+    Copyright 2014-2021, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -19,35 +19,21 @@
 
 */
 
-// approx timeout in seconds.
-#define SPLASH_TIME 3
-
-#include <sys/mman.h>
 #include <iostream>
 #include <string>
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <termios.h>
-#include <time.h>
-
-#include <map>
 #include <list>
+#include <termios.h>
 #include <pthread.h>
-#include <thread>
-#include <semaphore.h>
-#include <cstdio>
-#include <unistd.h>
+#include <atomic>
+
 #include <readline/readline.h>
 #include <readline/history.h>
 
-#include "Misc/Config.h"
+#include "Misc/CmdOptions.h"
 #include "Misc/SynthEngine.h"
 #include "MusicIO/MusicClient.h"
 #include "CLI/CmdInterface.h"
-#include "Interface/InterChange.h"
 #include "Misc/FileMgrFuncs.h"
-#include "Misc/FormatFuncs.h"
 
 #ifdef GUI_FLTK
     #include "MasterUI.h"
@@ -61,12 +47,14 @@
 extern map<SynthEngine *, MusicClient *> synthInstances;
 extern SynthEngine *firstSynth;
 
+// used by automated test launched via CLI
+std::atomic <bool> waitForTest{false};
+
 
 void mainRegisterAudioPort(SynthEngine *s, int portnum);
 int mainCreateNewInstance(unsigned int forceId);
 Config *firstRuntime = NULL;
-static int globalArgc = 0;
-static char **globalArgv = NULL;
+static std::list<std::string> globalAllArgs;
 bool isSingleMaster = false;
 bool bShowGui = true;
 bool bShowCmdLine = true;
@@ -88,7 +76,7 @@ void newBlock()
     {
         if ((firstRuntime->activeInstance >> i) & 1)
         {
-            while(configuring)
+            while (configuring)
                 usleep(1000);
             // in case there is still an instance starting from elsewhere
             configuring = true;
@@ -125,7 +113,7 @@ void yoshimiSigHandler(int sig)
             sigaction(SIGUSR1, &yoshimiSigAction, NULL);
             break;
         case SIGUSR2: // start next instance
-            if(isSingleMaster)
+            if (isSingleMaster)
                 newInstance();
             sigaction(SIGUSR2, &yoshimiSigAction, NULL);
             break;
@@ -134,103 +122,105 @@ void yoshimiSigHandler(int sig)
     }
 }
 
-#ifdef GUI_FLTK
-void do_start(void)
-{
-    std::string startup = YOSHIMI_VERSION;
-    startup = "Yoshimi V " + startup + " is starting";
-    int w = 300;
-    int h = 36;
-    int sx = (Fl::w() - w) / 2;
-    int sy = (Fl::h() - h) / 2;
-    Fl_Window window(sx, sy, w, h, "yoshimi start");
-    Fl_Box box(2, 2, w-4, h-4, startup.c_str());
-    box.box(FL_EMBOSSED_FRAME);
-    box.labelsize(16);
-    box.labelfont(FL_BOLD);
-    box.labelcolor(YOSHI_COLOUR);
-    window.end();
-    window.border(false);
 
-    for (int i = 0; i < 10; ++i)
-    {
-        window.show();
-        usleep(10000);
-        Fl::check();
-    }
-}
+static void *mainThread(void *arg)
+{
+#ifndef GUI_FLTK
+    bShowGui = false; // just to be sure
 #endif
-
-static void *mainGuiThread(void *arg)
-{
     sem_post((sem_t *)arg);
     map<SynthEngine *, MusicClient *>::iterator it;
 
 #ifdef GUI_FLTK
-    static bool first = true;
-    if (first)
-    {
-        first = false;
-        Fl::lock();
-    }
     const int textHeight = 15;
     const int textY = 10;
     const unsigned char lred = 0xd7;
     const unsigned char lgreen = 0xf7;
     const unsigned char lblue = 0xff;
+    int winH=10, winW=50;
+    int LbX=2, LbY=2, LbW=2, LbH=2;
+    int timeout =3;
+    std::string startup = YOSHIMI_VERSION;
+
+    if (bShowGui)
+    {
+        Fl::lock();
+
+        if (showSplash)
+        {
+            startup = "V " + startup;
+            winH = splashHeight;
+            winW = splashWidth;
+            LbX = 0;
+            LbY = winH - textY - textHeight;
+            LbW = winW;
+            LbH = textHeight;
+            timeout = 5;
+        }
+        else
+        {
+            startup = "Yoshimi V " + startup + " is starting";
+            winH = 36;
+            winW = 300;
+            LbX = 2;
+            LbY = 2;
+            LbW = winW - 4;
+            LbH = winH -4;
+            timeout = 3;
+        }
+    }//bShowGui
 
     Fl_PNG_Image pix("splash_screen_png", splashPngData, splashPngLength);
-    Fl_Window winSplash(splashWidth, splashHeight, "yoshimi splash screen");
-    Fl_Box box(0, 0, splashWidth,splashHeight);
-    box.image(pix);
-    std::string startup = YOSHIMI_VERSION;
-    startup = "V " + startup;
-    Fl_Box boxLb(0, splashHeight - textY - textHeight, splashWidth, textHeight, startup.c_str());
-    boxLb.box(FL_NO_BOX);
-    boxLb.align(FL_ALIGN_CENTER);
-    boxLb.labelsize(textHeight);
-    boxLb.labeltype(FL_NORMAL_LABEL);
-    boxLb.labelcolor(fl_rgb_color(lred, lgreen, lblue));
-    boxLb.labelfont(FL_HELVETICA | FL_BOLD);
-    winSplash.border(false);
-    if (splashSet && bShowGui && showSplash)
+    Fl_Window winSplash(winW, winH, "yoshimi splash screen");
+    Fl_Box box(0, 0, winW,winH);
+    Fl_Box boxLb(LbX, LbY, LbW, LbH, startup.c_str());
+
+    if (bShowGui)
     {
-        winSplash.position((Fl::w() - winSplash.w()) / 2, (Fl::h() - winSplash.h()) / 2);
+        if (showSplash)
+        {
+            box.image(pix);
+            boxLb.box(FL_NO_BOX);
+            boxLb.align(FL_ALIGN_CENTER);
+            boxLb.labelsize(textHeight);
+            boxLb.labeltype(FL_NORMAL_LABEL);
+            boxLb.labelcolor(fl_rgb_color(lred, lgreen, lblue));
+            boxLb.labelfont(FL_HELVETICA | FL_BOLD);
+        }
+        else
+        {
+            boxLb.box(FL_EMBOSSED_FRAME);
+            boxLb.labelsize(16);
+            boxLb.labelfont(FL_BOLD);
+            boxLb.labelcolor(YOSHI_COLOUR);
+        }
+        winSplash.border(false);
+        if (splashSet && bShowGui)
+        {
+            winSplash.position((Fl::w() - winSplash.w()) / 2, (Fl::h() - winSplash.h()) / 2);
+        }
+        else
+            splashSet = false;
+        do
+        {
+            winSplash.show();
+            usleep(33333);
+        }
+        while (firstSynth == NULL); // just wait
     }
-    else
-        splashSet = false;
-    do
+    else /* not bShowGui */
+#endif /*GUI_FLTK*/
     {
-            usleep(33333);
+        while (firstSynth == NULL); // just wait
     }
-#endif
-    while (firstSynth == NULL); // just wait
+
 
-#ifdef GUI_FLTK
-    GuiThreadMsg::sendMessage(firstSynth, GuiThreadMsg::NewSynthEngine, 0);
-#endif
     if (firstRuntime->autoInstance)
         newBlock();
     while (firstRuntime->runSynth)
     {
         firstRuntime->signalCheck();
-#ifdef GUI_FLTK
-        if (bShowGui)
-        {
-            if (splashSet)
-            {
-                winSplash.show();
-                usleep(1000);
-                if(time(&here_and_now) < 0) // no time?
-                    here_and_now = old_father_time + SPLASH_TIME;
-                if ((here_and_now - old_father_time) >= SPLASH_TIME)
-                {
-                    splashSet = false;
-                    winSplash.hide();
-                }
-            }
-        }
-#endif
+
         for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
         {
             SynthEngine *_synth = it->first;
@@ -264,14 +254,33 @@ static void *mainGuiThread(void *arg)
 #ifdef GUI_FLTK
             if (bShowGui)
             {
-                for (int i = 0; !_synth->getRuntime().LogList.empty() && i < 5; ++i)
+                MasterUI *guiMaster = _synth->getGuiMaster(false);
+                if (guiMaster)
                 {
-                    MasterUI *guiMaster = _synth->getGuiMaster(false);
-                    if (guiMaster)
-                    { guiMaster->Log(_synth->getRuntime().LogList.front());
-                        _synth->getRuntime().LogList.pop_front();
+                    if (guiMaster->masterwindow)
+                    {
+                        guiMaster->checkBuffer();
                     }
                 }
+                else
+                    GuiThreadMsg::processGuiMessages();
+                Fl::wait(33333);
+
+                if (splashSet)
+                {
+                    if (showSplash)
+                    {
+                        winSplash.show(); // keeps it in front;
+                    }
+                    if (time(&here_and_now) < 0) // no time?
+                        here_and_now = old_father_time + timeout;
+                    if ((here_and_now - old_father_time) >= timeout)
+                    {
+                        splashSet = false;
+                        winSplash.hide();
+                    }
+                }
+
             }
 #endif
         }
@@ -285,20 +294,16 @@ static void *mainGuiThread(void *arg)
             configuring = false;
             startInstance = testInstance; // to prevent repeats!
         }
-        else
-        {
-#ifdef GUI_FLTK
-            if (bShowGui)
-            {
-                Fl::wait(0.033333);
-                GuiThreadMsg::processGuiMessages();
-            }
-            else
-#endif
-                usleep(33333);
-        }
+        if (!bShowGui)
+            usleep(33333);
     }
 
+    if (waitForTest)
+        // After launching an automated test,
+        // get out of the way and leave main thread without persisting state...
+        // Test runs single threaded and we do not want to persist test state.
+        return NULL;
+
     if (firstRuntime->configChanged && (bShowGui | bShowCmdLine)) // don't want this if no cli or gui
         firstSynth->getRuntime().restoreConfig(firstSynth);
 
@@ -312,45 +317,39 @@ int mainCreateNewInstance(unsigned int forceId)
 {
     MusicClient *musicClient = NULL;
     unsigned int instanceID;
-    SynthEngine *synth = new SynthEngine(globalArgc, globalArgv, false, forceId);
+    SynthEngine *synth = new SynthEngine(globalAllArgs, LV2PluginTypeNone, forceId);
     if (!synth->getRuntime().isRuntimeSetupCompleted())
         goto bail_out;
     instanceID = synth->getUniqueId();
     if (!synth)
-    {
+    { // this has to be a direct call - no synth for log!
         std::cerr << "Failed to allocate SynthEngine" << std::endl;
         goto bail_out;
     }
 
     if (!(musicClient = MusicClient::newMusicClient(synth)))
     {
-        synth->getRuntime().Log("Failed to instantiate MusicClient");
+        synth->getRuntime().Log("Failed to instantiate MusicClient",_SYS_::LogError);
         goto bail_out;
     }
 
     if (!synth->Init(musicClient->getSamplerate(), musicClient->getBuffersize()))
     {
-        synth->getRuntime().Log("SynthEngine init failed");
+        synth->getRuntime().Log("SynthEngine init failed",_SYS_::LogError);
         goto bail_out;
     }
 
     if (!musicClient->Start())
     {
-        synth->getRuntime().Log("Failed to start MusicIO");
+        synth->getRuntime().Log("Failed to start MusicIO",_SYS_::LogError);
         goto bail_out;
     }
 #ifdef GUI_FLTK
     if (synth->getRuntime().showGui)
     {
         synth->setWindowTitle(musicClient->midiClientName());
-        if(firstSynth != NULL) //FLTK is not ready yet - send this message later for first synth
-        {
+        if (firstSynth != NULL) //FLTK is not ready yet - send this message later for first synth
             GuiThreadMsg::sendMessage(synth, GuiThreadMsg::NewSynthEngine, 0);
-        }
-        if (synth->getRuntime().audioEngine < 1)
-            fl_alert("Yoshimi can't find an available sound system. Running with no Audio");
-        if (synth->getRuntime().midiEngine < 1)
-            fl_alert("Yoshimi can't find an input system. Running with no MIDI");
     }
     else
         synth->getRuntime().toConsole = false;
@@ -380,7 +379,7 @@ int mainCreateNewInstance(unsigned int forceId)
 
 bail_out:
     synth->getRuntime().runSynth = false;
-    synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(");
+    synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(",_SYS_::LogError);
     if (musicClient)
     {
         musicClient->Close();
@@ -402,22 +401,12 @@ void *commandThread(void *) // silence warning (was *arg = NULL)
     return 0;
 }
 
-std::string runCommand(std::string command, bool clean)
+std::string runCommand(std::string command)
 {
-    const int lineLen = 63;
-    char returnLine[lineLen + 1];
-    FILE *fp = popen(command.c_str(), "r");
-    fgets(returnLine, lineLen, fp);
-    pclose(fp);
-    if (clean)
-    {
-        for (int i = 0; i < lineLen; ++ i)
-        {
-            if (returnLine[i] == ':')
-                returnLine[i] = '0';
-        }
-    }
-    return std::string(returnLine);
+    string returnLine = "";
+    file::cmd2string(command, returnLine);
+    //std::cout << returnLine << std::endl;
+    return returnLine;
 }
 
 int main(int argc, char *argv[])
@@ -431,12 +420,12 @@ int main(int argc, char *argv[])
      */
     std::string Home = getenv("HOME");
     std::string Config = file::loadText(Home + "/.config/yoshimi/yoshimi.config");
-    bool runGui = false;
     if (Config.empty())
     {
         std::cout << "Not there" << std::endl;
-        runGui = true;
+#ifdef GUI_FLTK
         showSplash = true;
+#endif
     }
     else
     {
@@ -445,15 +434,7 @@ int main(int argc, char *argv[])
         while (!Config.empty() && count < 16 && found < 3)
         { // no need to count beyond 16 lines!
             std::string line = func::nextLine(Config);
-            //std::cout << count << "  " << line  << std::endl;
             ++ count;
-
-            if (line.find("enable_gui") != std::string::npos)
-            {
-                ++ found;
-                if (line.find("yes") != std::string::npos)
-                    runGui = true;
-            }
             if (line.find("enable_splash") != std::string::npos)
             {
                 ++ found;
@@ -468,58 +449,45 @@ int main(int argc, char *argv[])
             }
         }
     }
+
     if (isSingleMaster)
     {
-        std::string firstText = runCommand("pgrep -o -x yoshimi", false);
-        int firstpid = std::stoi(firstText);
-        int firstTime = std::stoi(runCommand("ps -o etime= -p " + firstText, true));
-        int secondTime = std::stoi(runCommand("ps -o etime= -p " + std::to_string(getpid()), true));
 
+        std::string firstText = runCommand("pgrep -o -x yoshimi");
+        int firstpid = std::stoi(firstText);
+        int firstTime = std::stoi(runCommand("ps -o etimes= -p " + firstText));
+        int secondTime = std::stoi(runCommand("ps -o etimes= -p " + std::to_string(getpid())));
         if ((firstTime - secondTime) > 0)
         {
-                kill(firstpid, SIGUSR2); // this just sends a message
-                return 0;
+                kill(firstpid, SIGUSR2); // send message to 1st instance
+                return 0; // exit quietly
         }
     }
-#ifdef GUI_FLTK
-    bool useGui = false;
-    if (runGui)
-        useGui = true;
-    if (argc > 1)
-    {
-        for (int n = 1; n < argc; ++ n)
-            if (string(argv[n]) == "-i" || string(argv[n]) == "--no-gui" )
-            {
-                useGui = false; // overRide saved settings
-                break;
-            }
-            else if (string(argv[n]) == "-I" || string(argv[n]) == "--gui" )
-            {
-                useGui = true; // overRide saved settings
-                break;
-            }
-    }
-    if (useGui)
-        do_start();
 
-    bool guiStarted = false;
+    std::list<std::string> allArgs;
+
+    int gui;
+    int cmd;
+    CmdOptions(argc, argv, allArgs, gui, cmd);
+    globalAllArgs = allArgs;
+    bool mainThreadStarted = false;
+
+#ifdef GUI_FLTK
     time(&old_father_time);
     here_and_now = old_father_time;
 #endif
 
-
     struct termios  oldTerm;
     tcgetattr(0, &oldTerm);
 
     std::cout << "Yoshimi " << YOSHIMI_VERSION << " is starting" << std::endl; // guaranteed start message
-    globalArgc = argc;
-    globalArgv = argv;
+
     bool bExitSuccess = false;
     map<SynthEngine *, MusicClient *>::iterator it;
 
-    pthread_t thr;
-    pthread_attr_t attr;
-    sem_t semGui;
+    pthread_t threadMainLoop;
+    pthread_attr_t pthreadAttr;
+    sem_t semMain;
 
     if (mainCreateNewInstance(0) == -1)
     {
@@ -537,78 +505,96 @@ int main(int argc, char *argv[])
 
         std::cout << "\nExisting config older than " << MIN_CONFIG_MAJOR << "." << MIN_CONFIG_MINOR << "\nCheck settings, save and restart.\n"<< std::endl;
     }
-    if(sem_init(&semGui, 0, 0) == 0)
+    if (sem_init(&semMain, 0, 0) == 0)
     {
-        if (pthread_attr_init(&attr) == 0)
+        if (pthread_attr_init(&pthreadAttr) == 0)
         {
-            if (pthread_create(&thr, &attr, mainGuiThread, (void *)&semGui) == 0)
-            {
-#ifdef GUI_FLTK
-                guiStarted = true;
-#endif
-            }
-            pthread_attr_destroy(&attr);
+            if (pthread_create(&threadMainLoop, &pthreadAttr, mainThread, (void *)&semMain) == 0)
+                mainThreadStarted = true;
+            pthread_attr_destroy(&pthreadAttr);
         }
     }
-#ifdef GUI_FLTK
-    if (!guiStarted)
+
+    if (!mainThreadStarted)
     {
-        std::cout << "Yoshimi can't start main gui loop!" << std::endl;
+        std::cout << "Yoshimi can't start main loop!" << std::endl;
         goto bail_out;
     }
-    sem_wait(&semGui);
-    sem_destroy(&semGui);
-#endif
+    sem_wait(&semMain);
+    sem_destroy(&semMain);
+
     memset(&yoshimiSigAction, 0, sizeof(yoshimiSigAction));
     yoshimiSigAction.sa_handler = yoshimiSigHandler;
     if (sigaction(SIGUSR1, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGUSR1 handler failed");
+        firstRuntime->Log("Setting SIGUSR1 handler failed",_SYS_::LogError);
     if (sigaction(SIGUSR2, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGUSR2 handler failed");
+        firstRuntime->Log("Setting SIGUSR2 handler failed",_SYS_::LogError);
     if (sigaction(SIGINT, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGINT handler failed");
+        firstRuntime->Log("Setting SIGINT handler failed",_SYS_::LogError);
     if (sigaction(SIGHUP, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGHUP handler failed");
+        firstRuntime->Log("Setting SIGHUP handler failed",_SYS_::LogError);
     if (sigaction(SIGTERM, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGTERM handler failed");
+        firstRuntime->Log("Setting SIGTERM handler failed",_SYS_::LogError);
     if (sigaction(SIGQUIT, &yoshimiSigAction, NULL))
-        firstRuntime->Log("Setting SIGQUIT handler failed");
+        firstRuntime->Log("Setting SIGQUIT handler failed",_SYS_::LogError);
     // following moved here for faster first synth startup
     firstSynth->loadHistory();
     firstSynth->installBanks();
+#ifdef GUI_FLTK
+    if (bShowGui)
+        GuiThreadMsg::sendMessage(firstSynth, GuiThreadMsg::NewSynthEngine, 0);
+#endif
 
     //create command line processing thread
-    pthread_t cmdThr;
+    pthread_t threadCmd;
 
-    if(bShowCmdLine)
+    if (bShowCmdLine)
     {
-        if (pthread_attr_init(&attr) == 0)
+        if (pthread_attr_init(&pthreadAttr) == 0)
         {
-            if (pthread_create(&cmdThr, &attr, commandThread, (void *)firstSynth) == 0)
+            if (pthread_create(&threadCmd, &pthreadAttr, commandThread, (void *)firstSynth) == 0)
             {
                 ;
             }
-            pthread_attr_destroy(&attr);
+            pthread_attr_destroy(&pthreadAttr);
         }
     }
 
-    void *ret;
-    pthread_join(thr, &ret);
-    if(ret == (void *)1)
+//firstRuntime->Log("test normal msg");
+//firstRuntime->Log("test not serious",_SYS_::LogNotSerious);
+//firstRuntime->Log("test error msg",_SYS_::LogError);
+//firstRuntime->Log("test not serious error",_SYS_::LogNotSerious | _SYS_::LogError);
+
+    void *res;
+    pthread_join(threadMainLoop, &res);
+    if (res == (void *)1)
     {
         goto bail_out;
     }
+    if (waitForTest && threadCmd)
+    {   // CLI about to launch TestRunner for automated test
+        MusicClient* music = synthInstances[firstSynth];
+        if (music) music->Close(); // causes esp. JackClient to detach
+
+        pthread_join(threadCmd, &res);
+        if (res == (void *)1)
+        {
+            goto bail_out;
+        }
+    }
     bExitSuccess = true;
 
 bail_out:
+    // firstSynth is freed in the for loop below, so save this for later
+    int exitType = firstSynth->getRuntime().exitType;
     for (it = synthInstances.begin(); it != synthInstances.end(); ++it)
     {
         SynthEngine *_synth = it->first;
         MusicClient *_client = it->second;
         _synth->getRuntime().runSynth = false;
-        if(!bExitSuccess)
+        if (!bExitSuccess)
         {
-            _synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(");
+            _synth->getRuntime().Log("Bail: Yoshimi stages a strategic retreat :-(",_SYS_::LogError);
         }
 
         if (_client)
@@ -623,16 +609,15 @@ bail_out:
             delete _synth;
         }
     }
-    if(bShowCmdLine)
+    if (bShowCmdLine)
         tcsetattr(0, TCSANOW, &oldTerm);
     if (bExitSuccess)
     {
-        int type = firstSynth->getRuntime().exitType;
-        if (type == FORCED_EXIT)
+        if (exitType == FORCED_EXIT)
             std::cout << "\nExit was forced :(" << std::endl;
         else
             std::cout << "\nGoodbye - Play again soon?"<< std::endl;
-        exit(type);
+        exit(exitType);
     }
     else
         exit(EXIT_FAILURE);
